package gov.nih.nlm.nls.lexCheck.install.utils;

import java.util.*;
import java.io.*;

/**
* FileReplace allows multiple transformations on a file before
* writing it back out. 
* 
*  It's supposed to work something like this:
*  
*   FileReplace aFile = new FileReplace( fileName );
*
*   aFile.changeFile("__MMTX_USER", "FRED");
*   aFile.changeFile("__PASSWORD", "foo");
*       . 
*       . 
*       . 
*
*   aFile.writeFile();
* 
* There is also a static method that combines all of these
*  FileReplace.replace( pFileName, oldPattern, newPattern );
*/
public final class FileReplace 
{
	/**
	* This is a constructor for FileReplace
	*
	* @param String pFileName (The full pathname)
	*/
  	public FileReplace( String pFileName )
  	{
    	this.originalFileName = pFileName;

    	// Read in the contents of the file
    	readFile( this.originalFileName );
  	} //*** End Constructor FileReplace

	/**
	* Method readFile reads the contents of the file 
	* 
	* @param String pFileName
	*/
	public void readFile(String pFileName)
  	{
    	try
		{ 
		  BufferedReader fStream = 
			new BufferedReader(new FileReader(pFileName));
		  this.fileContents = new Vector<String>();
      
      		try 
			{
				String line = null;
				while ((line = fStream.readLine()) != null )  
				{
				  // ----------------------------
				  // read in each line of the table 
				  // ----------------------------
				  this.fileContents.addElement( line );
				}
      		} 
			catch ( Exception e5 ) 
			{
				System.out.println(
					"Problem reading in the contents of the file " 
					+ pFileName + " : " + e5.toString() );
      		}
      
			// close the source table 
			try 
			{
				fStream.close();
      		}
			catch ( Exception e4 ) 
			{
				System.out.println("Not able to close the file: " 
					+ e4.toString());
      		}
      		fStream = null;
      
      } catch ( Exception e3 ) {
	System.out.println("Not able to open file: " + e3.toString());
      }
   } //*** End public void readFile(String pFileName)
/**
 * Method changeFile changes all occurrences of oldPattern to newPattern
 * in the file 
 * 
 * @param String oldPattern	
 * @param  String newPattern
 * 
*/
public void changeFile(String oldPattern, String newPattern)
  { 
    String line = null;
    String changedLine = null;
    for ( int i = 0; i < this.fileContents.size(); i++ ) {
      line = this.fileContents.get(i);
      changedLine = changeLine( line, oldPattern, newPattern);
      this.fileContents.set(i, changedLine);
    }

   } //*** End public void changeFile(String oldPattern, StringNewPattern)

/**
 * Method writeFile creates a new file with the updated contents of the file.
 * 
 * @param String newFileName
*/
public void writeFile( String newFileName )
  { 
    // -------------------------
    // Create a new file
    // -------------------------
    
    try { 
      PrintWriter fStream = new PrintWriter (new BufferedWriter(new FileWriter( newFileName )));
      
      for ( int i = 0 ; i < this.fileContents.size(); i++ ) 
	fStream.println(this.fileContents.get(i) );
      
      fStream.close();
      
      fStream = null;
      
    } catch ( Exception e ) {
      System.out.println( "Not able to write to " + newFileName );
    }

   } //*** End public void writeFile()

/**
 * Method writeFile replaces the original file with the file with updated contents
*/
public void writeFile()
  { 
    writeFile( originalFileName );
   } //*** End public void writeFile()

	/**
	* Method getFileContents returns the contents of the file 
	* 
	* @returns Vector of String
	*/
public Vector<String> getFileContents()
  { 
    return ( this.fileContents );
   } //*** End public void getFileContents()

/**
 * Method changeLine 
 * 
 * @param String pLine		
 * @param  String oldPattern	
 * @param  String newPattern	
 * @returns	String
*/
public String changeLine(String pLine, String oldPattern, String newPattern)
  { 
    String     returnValue = null;

    // -------------------------------------------
    // Look for instance of "___MMTX" in the line
    // -------------------------------------------
    int i = pLine.indexOf( oldPattern );

    if ( i > -1 ) {
      
      String partA = pLine.substring( 0,i );
      String partC = pLine.substring( i + oldPattern.length() );

      returnValue = partA + newPattern + partC;

      partA = null;
      partC = null;

      
    } else 

      returnValue = pLine;
    
    return( returnValue );

   }

/**
 * Method replace replaces the contents of the oldFile with updated contents, 
 * replacing all instances of oldPattern with newPattern. It then writes
 * this content out to newFileName
 * 
 * @param String oldFileName		
 * @param String newFileName		
 * @param  String oldPattern		
 * @param  String newPattern		
*/
public static void replace(String oldFileName, 
			   String newFileName,
			   String oldPattern, 
			   String newPattern)
  { 
    FileReplace aFile = new FileReplace( oldFileName);
    aFile.changeFile( oldPattern, newPattern );
    aFile.writeFile( newFileName );

    aFile = null;

   }

/**
 * Method replace replaces the contents of the fileName with updated contents, 
 * replacing all instances of oldPattern with newPattern. It then writes
 * this content back out to FileName
 * 
 * @param String oldFileName		
 * @param  String oldPattern		
 * @param  String newPattern		
 * 
*/
public static void replace(String oldFileName, String oldPattern, 
			   String newPattern)
  {
    FileReplace aFile = new FileReplace( oldFileName);
    aFile.changeFile( oldPattern, newPattern );
    aFile.writeFile();

    aFile = null;
   }

/**
 * This is a test main
 * 
 * @param String argv[] 	The command line input, tokenized
 *
 * @return int  0|-1		0 is returned if no problems, -1 is
 * 				is returned if there is a problem.
 *
*/
  public final static void main( String argv[] )
  {
    int returnValue = 0;

    if ((argv.length > 0 ) && ( argv[0].equals("-h") ))
    {
       usage();
    }
    else
    {
      String oldFileName = null; 
      String newFileName =  null;
      String oldPattern  =  null;
      String newPattern  =  null;
      
      if ( argv.length == 3 ) {
	oldFileName = argv[0];
	oldPattern  = argv[1];
	newPattern  = argv[2];
	FileReplace.replace( oldFileName, oldPattern, newPattern );

      } else {
	oldFileName = argv[0];
	newFileName = argv[1];
	oldPattern  = argv[2];
	newPattern  = argv[3];
	FileReplace.replace( oldFileName, newFileName, oldPattern, newPattern );
      }
    }	

    System.exit( returnValue );

   } //*** End main ( String argv[] )

/**
 * Every main should have one. Why? I can't remember. Right! 
 * because I cannot remember the reason or function behind the 5000+
 * functions I have written over the last 6 years, I need some help
 * reminding me what this thing is supposed to do.
 * 
 * The usage has got to be the first thing done before writing any 
 * code. 
 *
 * @Example call
 * 
 *	_usage();
 *	
 *
*/
  private static final void usage()
  {

    System.out.println("java utils.FileReplace oldFileName [newFileName] oldPattern newPattern");
    System.out.println("java utils.FileReplace [-h]" );
    System.out.println("			-h prints out the help");
    
    System.out.println(" Filereplace replaces the contents of the oldFileName with updated contents, ");
    System.out.println(" replacing all instances of oldPattern with newPattern. It then writes");
    System.out.println(" this content back out to newFilename if it was specified, otherwise it");
    System.out.println(" writes the updated contents back to oldFileName. ");
 
    } //*** End usage()

  // Private Declarations
     private Vector<String> fileContents = null;
     private String originalFileName = null; 

}
