package gov.nih.nlm.nls.lexCheck.install.utils;

import java.text.*;
import java.io.*;
import java.lang.Process;
import java.lang.StringBuffer;

/**
* 
* U is a helper static class to make newlines, path separators, and the
* like as short and unobtrusive a call in the source code as possible.
* 
* Description: java source file. <p>
* NewLines are different on different platforms and the call to deterine
* them is obtrusive.  This static call should make it easier to do
* and in doing so, should eliminate the temptation to just code a "\n"
* and hope that this does not get run on an NT. 
* <p>
* History
* <p>
*       Fri Jun 06 10:46:45 EDT 2000,     divita     Initial Version
* <p>
* Notes: 
* <p>
*/

public class U 
{
  public static final String NL = 
	System.getProperty("line.separator").toString(); 
  public static final String FS = 
	System.getProperty("file.separator").toString();  
  public static final String PS = 
	System.getProperty("path.separator").toString(); 
  
  private static BufferedReader standardInput = null;
  private static Runtime runtime = null; 
  
  /**
   * Method openStandardInput
   * 
   * @returns BufferedReader  
   */
  public static BufferedReader openStandardInput()
  { 
    
    if ( standardInput == null ) { 
      
      // Open the standard input to get messages
      try {
	standardInput = new BufferedReader(new InputStreamReader(System.in));
      } catch ( Exception e3 ) {
	System.err.println("Not able to open the standard input for reading");
      }
    }
    
    return ( standardInput );
  }
    
/**
 * Given two different milliseconds (long) compkute the hours, minutes, and 
 * seconds and then print that into a string to be returned. 
 *
 * @Example call
 * 
 *	formatDiff(begin, end);
 *
 * @param  long begin		Time in milliseconds when activity began
 * @param  long end		Time in milliseconds when activity ended
*/
  public static final String formatDiff(long begin, long end)
  {
    String     returnValue = null;
    int hours, minutes, milliseconds;
    long diff;
    double seconds;
    DecimalFormat df = new DecimalFormat ( "#0.00" );

    diff = end - begin;
    seconds = (diff * 1.0) / 1000.0;
    hours = (int) (seconds / 3600);
    seconds -= (hours * 3600) * 1.0;
    minutes = (int) (seconds / 60);
    seconds -= (minutes * 60) * 1.0;
    returnValue = hours + ":" + minutes + ":" + 
                  df.format(seconds);

    return( returnValue );
  } //*** End formatDifff


  /**
   * Method isWindows returns true if this platform is a windows platform
   *
   * @returns boolean r
   * 
   */
  public static final boolean isWindows() 
  { 
    boolean returnCode = false;
    String osName = System.getProperty("os.name");
    
    if ( osName.toLowerCase().indexOf("windows") > -1 ) 
      returnCode = true;
    
    return ( returnCode );
  } 

  /**
   * Method run 
   * @param String command 
   * 
   * @returns String Anything that comes back from that command
   */
  public static String run(String pCommand ) 
  { 
    Process p = null; 
    StringBuffer buff = new StringBuffer();

    try {
      if ( runtime == null )
	runtime = Runtime.getRuntime();
      
      if ( runtime != null ) 
	p = runtime.exec( pCommand.trim() );

	
      try { 
	BufferedReader outputStream = new BufferedReader(
					new InputStreamReader( p.getInputStream()));
	
	String line = null;
	while ( (line = outputStream.readLine()) != null ) {
	  buff.append( line );
	  buff.append( U.NL );
	} 
	outputStream.close();

	
      } catch ( Exception e2 ) { 
	System.err.println("Problem with getting output from the command " 
			   + U.NL +
			   pCommand + U.NL +
			   e2.toString());
      }
      
    } catch ( Exception e ) { 
      System.err.println("Problem running the command |"  +
			 pCommand + "| " + e.getMessage() + "|" + e.toString() );  
      e.printStackTrace();
  
    }
    p.destroy();
    p = null;

    return (buff.toString());
  }  
  
/**
 * Method quoted returns the string correctly quoted for use within a sql statement.
 * That is, if the string contains double quotes, single quotes will be added to
 * the returned string. If single quotes are in the term, double quotes will be added. 
 * 
 * @param  String term 		
 * @returns			String
*/
public static String quoted( String term )
  { 
    StringBuffer buff = new StringBuffer();
    buff.append( DOUBLE_QUOTE );
    buff.append( escaped(term) );
    buff.append(  DOUBLE_QUOTE ) ;

    return( buff.toString() );
   }

/* 
 * Method escaped returns the string correctly escaped for use within a 
 * sql statement. That is, if the string contains double quotes, 
 * escape sequences will be added to the returned string.
 * 
 * @param  String term 		
 * @returns			String
*/
public static String escaped( String term ) 
  { 
    String     returnValue = term;
    String     tmp = term;
    int from, to;

    // First let's escape all occurrences of \ (backslash) in text.

    if ( term.indexOf( '\\' ) > -1 ) {
      to = 0;
      from = 0;
      tmp = "";

      while ((to = term.indexOf('\\', from)) >= 0)
      {
         tmp += term.substring(from, to) + "\\\\";
         from = to + 1;
      } // while

      tmp += term.substring(from);
    } // fi

    // Now handle the double quotes (").

    if ( tmp.indexOf( DOUBLE_QUOTE ) > -1 ) {
      to = 0;
      from = 0;
      returnValue = "";

      while ((to = tmp.indexOf(DOUBLE_QUOTE, from)) >= 0)
      {
         returnValue += tmp.substring(from, to) + "\\" + "\"";
         from = to + 1;
      } // while

      returnValue += tmp.substring(from);
    } // fi
    else
       returnValue = tmp;

    return( returnValue );

   }

  private static final char               QUOTE = '\'';
  private static final char        DOUBLE_QUOTE = '"';
  
  } // End of the Class U
