package gov.nih.nlm.nls.lexCheck.Api;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Gram.*;
/*****************************************************************************
* This class provides an Java Api for reading lexical records in text format 
* and send dictionary variants (information) to output.  
* The input can be a file or a string.
*
* <p><b>History:</b>
*
* @author chlu
*
* @version    V-2019
****************************************************************************/
public class ToDicVarsApi
{
    // public method
    /**
    * A method to generate inflection variables from a file contains
    * lexical records in text format.
    *
    * @param inFile  the file name of the lexical records in text format
    *
    * @return  a vector of elements of InflVar
    */
    public static Vector<DicVar> GetDicVarsFromTextFile(String inFile)
    {
        CheckSt st = new CheckSt();     // upper level  Check state
        CheckSt catSt = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
            // Category dependant Check state
        LineObject lineObject = new LineObject();
        Vector<DicVar> dicVars = new Vector<DicVar>();
        try
        {
            BufferedReader inReader = new BufferedReader(new InputStreamReader(
                new FileInputStream(inFile), "UTF-8"));
            while(lineObject != null)
            {
                // check if read in the next line
                if(lineObject.IsGoToNext() == true)
                {
                    lineObject.SetLine(inReader.readLine());
                    lineObject.IncreaseLineNum();
                }
                if(lineObject.GetLine() == null)
                {
                    break;
                }
                // go through each line
                // check input for quitiing
                boolean flag = 
                    CheckGrammer.Check(lineObject, true, st, catSt, false);
                // generate dicVars when processing one lexical record 
                if((st.GetCurState() == CheckSt.CHECK_BASE)
                && (flag == true))
                {
					LexRecord lexRecord = CheckGrammer.GetLexRecord();
                    InflVarsAndAgreements infls = 
                        new InflVarsAndAgreements(lexRecord);
					Vector<InflVar> inflVars = infls.GetInflValues();
					for(InflVar inflVar:inflVars)
					{
						String var = inflVar.GetVar();
						String cat = inflVar.GetCat();
						String infl = inflVar.GetInflection();
						String eui = inflVar.GetEui();
						boolean unique = inflVar.GetUnique();
						// check if acrAbb
						boolean acrAbbFlag = false;
						if((lexRecord.GetAcronyms().size() > 0)
						|| (lexRecord.GetAbbreviations().size() > 0))
						{
							acrAbbFlag = true;
						}
						// check if proper noun
						boolean properNounFlag = false;
						if((cat.equals("noun") == true)
						&& (lexRecord.GetCatEntry().GetNounEntry().IsProper() == true))
						{
							properNounFlag = true;
						}
						// check if has spVar
						boolean spVarFlag = false;
						if(lexRecord.GetSpellingVars().size() > 0)
						{
							spVarFlag = true;
						}
						DicVar dicVar = new DicVar(var, cat, infl, eui, unique,
							acrAbbFlag, properNounFlag, spVarFlag);
						dicVars.add(dicVar);
					}
                }
            }
            inReader.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return dicVars;
    }
    /**
    * A unit test driver for ToXmlApi.
	* 
	* @param  args  input arguments 
    */
    public static void main(String[] args)
    {
        String inFile = "molt"; 
        // Go through LexCheck
        Vector<DicVar> temp = ToDicVarsApi.GetDicVarsFromTextFile(inFile);
    }
}
