package gov.nih.nlm.nls.lexCheck.Cat.Auxi;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
/*****************************************************************************
* This class checks inflectional variants for syntactical category of auxilary.
*
* <p><b>History:</b>
*
* @author NLM LVG Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckFormatAuxVariant implements CheckFormat
{
    // public method
    /**
    * Check a line to see if it is a legel infelctional variants of auxilary.
    *
    * @param   filler   the filler of inflectional variants of auxilary.
    *
    * @return  true or false for legal and illegal filler.
    */
    public boolean IsLegalFormat(String filler)
    {
        boolean flag = false;
        StringTokenizer buf = new StringTokenizer(filler, ";(,):");
        String inflection = new String();
        if(buf.hasMoreTokens() == true)
        {
            inflection = buf.nextToken();
        }
        else
        {
            return false;
        }
        // get tense code
        String tenseCode = new String();
        if(buf.hasMoreTokens() == true)
        {
            tenseCode = buf.nextToken();
        }
        else
        {
            return false;
        }
        // check format based on tense code
        if(tenseCode_.contains(tenseCode) == true)
        {
            if(tenseCode.startsWith(TC_INFINITIVE))
            {
                flag = tenseCode.equals(TC_INFINITIVE);
            }
            else if((tenseCode.startsWith(TC_PAST_PART) == true)
            || (tenseCode.startsWith(TC_PRES_PART) == true))
            {
                String negative = new String();
                if(buf.hasMoreTokens() == true)
                {
                    negative = buf.nextToken();
                    flag = negative.equals(NEGATIVE_CODE);
                }
                else
                {
                    flag = true; 
                }
            }
            else if((tenseCode.startsWith(TC_PAST) == true)
            || (tenseCode.startsWith(TC_PRES) == true))
            {
                String feature = new String();
                if(buf.hasMoreTokens() == true)
                {
                    feature = buf.nextToken();
                    if(feature.equals(NEGATIVE_CODE) == true)
                    {
                        flag = true;
                    }
                    else
                    {
                        if(agreementFeature_.contains(feature) == true)
                        {
                            flag = true;
                            while(buf.hasMoreTokens() == true)
                            {
                                feature = buf.nextToken();
                                flag = ((feature.equals(NEGATIVE_CODE))
                                || (agreementFeature_.contains(feature)));
                                if(flag == false)
                                {
                                    break;
                                }
                            }
                        }
                        else
                        {
                            flag = false;
                        }
                    }
                }
                else
                {
                    flag = true;
                }
            }
        }
        else
        {
            return false;
        }
        return flag;
    }
    // data member
    private final static int LEGAL_TENSE_CODE_NUM = 5;
    private final static int LEGAL_AGREEMENT_FEATURE_NUM = 8;
    private final static String NEGATIVE_CODE = "negative";
    // tense code
    private final static String TC_PAST = "past";
    private final static String TC_PRES = "pres";
    private final static String TC_PAST_PART = "past_part";
    private final static String TC_PRES_PART = "pres_part";
    private final static String TC_INFINITIVE = "infinitive";
    // Agreement features
    private final static String AF_FST_SING = "fst_sing";
    private final static String AF_FST_PLUR = "fst_plur";
    private final static String AF_SECOND = "second";
    private final static String AF_SEC_SING = "sec_sing";
    private final static String AF_SEC_PLUR = "sec_plur";
    private final static String AF_THIRD = "third";
    private final static String AF_THR_SING = "thr_sing";
    private final static String AF_THR_PLUR = "thr_plur";
    private static HashSet<String> tenseCode_ 
        = new HashSet<String>(LEGAL_TENSE_CODE_NUM);
    private static HashSet<String> agreementFeature_ = 
        new HashSet<String>(LEGAL_AGREEMENT_FEATURE_NUM);
    static
    {
        tenseCode_.add(TC_PAST);
        tenseCode_.add(TC_PRES);
        tenseCode_.add(TC_PAST_PART);
        tenseCode_.add(TC_PRES_PART);
        tenseCode_.add(TC_INFINITIVE);
        agreementFeature_.add(AF_FST_SING);
        agreementFeature_.add(AF_FST_PLUR);
        agreementFeature_.add(AF_SECOND);
        agreementFeature_.add(AF_SEC_SING);
        agreementFeature_.add(AF_SEC_PLUR);
        agreementFeature_.add(AF_THIRD);
        agreementFeature_.add(AF_THR_SING);
        agreementFeature_.add(AF_THR_PLUR);
    }
}
