package gov.nih.nlm.nls.lexCheck.Cat.Noun;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.CkLib.*;
/*****************************************************************************
* This class checks filler for syntactical category of noun.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-58: chlu, 11/20/18, add a new field class_type - check syntax
* </ul>
*
* @author NLM LVG Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckNoun
{
    // public method
    /**
    * Check a line to see if it is a legel filler.
    *
    * @param   lineObject  the java object of a input line
    * @param   printFlag   the flag for print out check result
    * @param   catSt       the state of category
    * @param   lexObj   the current object of lexical record
    * @param   debugFlag   the flag for print out debug information
    *
    * @return  true or false for legal and illegal filler.
    */
    public static boolean Check(LineObject lineObject, boolean printFlag,
        CheckSt catSt, LexRecord lexObj, boolean debugFlag)
    {
        boolean flag = true;
        int curSt = catSt.GetCurState();
        // reset the state if it is first time
        if(curSt == CheckSt.CHECK_CAT_ENTRY)
        {
            curSt = CheckSt.CHECK_NOUN_VARIANTS;
            catSt.SetCurState(curSt);
        }
        switch(curSt)
        {
            case CheckSt.CHECK_NOUN_VARIANTS:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkVariants_, new UpdateNounVariants(), 
                    CheckCode.FQ_ONE_MANY, true);
                PrintStep(CheckSt.CHECK_NOUN_VARIANTS, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_NOUN_COMPL:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkCompl_, new UpdateNounCompl(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_NOUN_COMPL, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_NOUN_NOMINALIZATION:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkNominalization_, new UpdateNounNominalization(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_NOUN_NOMINALIZATION, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_NOUN_PROPER:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkProper_, new UpdateNounProper(), 
                    CheckCode.FQ_ZERO_ONE_WHOLE_LINE, false);
                PrintStep(CheckSt.CHECK_NOUN_PROPER, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_NOUN_TRADENAME:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkTradeName_, new UpdateNounTradeName(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_NOUN_TRADENAME, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_NOUN_TRADEMARK:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkTradeMark_, new UpdateNounTradeMark(), 
                    CheckCode.FQ_ZERO_ONE_WHOLE_LINE, false);
                PrintStep(CheckSt.CHECK_NOUN_TRADEMARK, debugFlag, 
                    lineObject.GetLine());
                break;
        }
        return flag;
    }
    // private method
    private static void PrintStep(int state, boolean debugFlag, String line)
    {
        if(debugFlag == true)
        {
            switch(state)
            {
                case CheckSt.CHECK_NOUN_VARIANTS:
                    System.out.println("-- Checked Noun Variants: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_NOUN_COMPL:
                    System.out.println("-- Checked Noun Complement: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_NOUN_NOMINALIZATION:
                    System.out.println("-- Checked Noun Nominalization: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_NOUN_PROPER:
                    System.out.println("-- Checked Noun Proper: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_NOUN_TRADENAME:
                    System.out.println("-- Checked Noun Trade Name: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_NOUN_TRADEMARK:
                    System.out.println("-- Checked Noun Trade Mark: '" 
                        + line + "'");
                    break;
            }
        }
    }
    // data member
    private static CheckObject checkVariants_ = null;
    private static HashSet<String> variantsNextStartStrs_ 
        = new HashSet<String>(10);
    private static CheckObject checkCompl_ = null;
    private static HashSet<String> complNextStartStrs_ 
        = new HashSet<String>(9);
    private static CheckObject checkNominalization_ = null;
    private static HashSet<String> nominalizationNextStartStrs_ 
        = new HashSet<String>(8);
    private static CheckObject checkProper_ = null;
    private static HashSet<String> properNextStartStrs_ 
        = new HashSet<String>(7);
    private static CheckObject checkTradeName_ = null;
    private static HashSet<String> tradeNameNextStartStrs_ 
        = new HashSet<String>(6);
    private static HashSet<String> tradeNameNextLine_ 
        = new HashSet<String>(1);
    private static CheckObject checkTradeMark_ = null;
    private static HashSet<String> tradeMarkNextStartStrs_ 
        = new HashSet<String>(5);
    static
    {
        // slot variants=
        variantsNextStartStrs_.add("\tcompl=");
        variantsNextStartStrs_.add("\tnominalization_of=");
        variantsNextStartStrs_.add("\ttrademark=");
        variantsNextStartStrs_.add("\ttrademark");
        variantsNextStartStrs_.add("\tproper");
        variantsNextStartStrs_.add("\tacronym_of=");
        variantsNextStartStrs_.add("\tabbreviation_of=");
        variantsNextStartStrs_.add("\tclass_type=");
        variantsNextStartStrs_.add("annotation=");
        variantsNextStartStrs_.add("signature=");
        variantsNextStartStrs_.add("}");
        checkVariants_ = new CheckObject("\tvariants=", 
            ErrMsg.NOUN_VAR_SLOT_ERR, ErrMsg.NOUN_VAR_FILLER_ERR, 
            CheckSt.CHECK_NOUN_COMPL, variantsNextStartStrs_,
            new CheckFormatNounVariants());
        // slot compl=
        complNextStartStrs_.add("\tnominalization_of=");
        complNextStartStrs_.add("\ttrademark=");
        complNextStartStrs_.add("\ttrademark");
        complNextStartStrs_.add("\tproper");
        complNextStartStrs_.add("\tacronym_of=");
        complNextStartStrs_.add("\tabbreviation_of=");
        complNextStartStrs_.add("\tclass_type=");
        complNextStartStrs_.add("annotation=");
        complNextStartStrs_.add("signature=");
        complNextStartStrs_.add("}");
        checkCompl_ = new CheckObject("\tcompl=", 
            ErrMsg.NOUN_COMPL_SLOT_ERR, ErrMsg.NOUN_COMPL_FILLER_ERR,
            CheckSt.CHECK_NOUN_NOMINALIZATION, complNextStartStrs_, 
            new CheckFormatNounCompl());
        // slot nominalization=
        nominalizationNextStartStrs_.add("\tproper");
        nominalizationNextStartStrs_.add("\ttrademark=");
        nominalizationNextStartStrs_.add("\ttrademark");
        nominalizationNextStartStrs_.add("\tacronym_of=");
        nominalizationNextStartStrs_.add("\tabbreviation_of=");
        nominalizationNextStartStrs_.add("\tclass_type=");
        nominalizationNextStartStrs_.add("annotation=");
        nominalizationNextStartStrs_.add("signature=");
        nominalizationNextStartStrs_.add("}");
        checkNominalization_ = new CheckObject("\tnominalization_of=", 
            ErrMsg.NOUN_NOMINALIZATION_SLOT_ERR, 
            ErrMsg.NOUN_NOMINALIZATION_FILLER_ERR, 
            CheckSt.CHECK_NOUN_PROPER, nominalizationNextStartStrs_, 
            new CheckFormatNounNominalization());
        // slot proper
        properNextStartStrs_.add("\ttrademark=");
        properNextStartStrs_.add("\ttrademark");
        properNextStartStrs_.add("\tacronym_of=");
        properNextStartStrs_.add("\tabbreviation_of=");
        properNextStartStrs_.add("\tclass_type=");
        properNextStartStrs_.add("annotation=");
        properNextStartStrs_.add("signature=");
        properNextStartStrs_.add("}");
        checkProper_ = new CheckObject("\tproper", 
            ErrMsg.NOUN_PROPER_SLOT_ERR, ErrMsg.NONE,
            CheckSt.CHECK_NOUN_TRADENAME, properNextStartStrs_, null);
        // slot trademark=    (tradename)
        tradeNameNextStartStrs_.add("\ttrademark");
        tradeNameNextStartStrs_.add("\tacronym_of=");
        tradeNameNextStartStrs_.add("\tabbreviation_of=");
        tradeNameNextStartStrs_.add("\tclass_type=");
        tradeNameNextStartStrs_.add("annotation=");
        tradeNameNextStartStrs_.add("signature=");
        tradeNameNextStartStrs_.add("}");
        tradeNameNextLine_.add("\ttrademark");
        checkTradeName_ = new CheckObject("\ttrademark=", 
            ErrMsg.NOUN_TRADENAME_SLOT_ERR, 
            ErrMsg.NOUN_TRADENAME_FILLER_ERR, 
            CheckSt.CHECK_NOUN_TRADEMARK, tradeNameNextStartStrs_, 
            tradeNameNextLine_, null);
        // slot trademark
        tradeMarkNextStartStrs_.add("\tacronym_of=");
        tradeMarkNextStartStrs_.add("\tabbreviation_of=");
        tradeMarkNextStartStrs_.add("\tclass_type=");
        tradeMarkNextStartStrs_.add("annotation=");
        tradeMarkNextStartStrs_.add("signature=");
        tradeMarkNextStartStrs_.add("}");
        checkTradeMark_ = new CheckObject("\ttrademark", 
            ErrMsg.NOUN_TRADEMARK_SLOT_ERR, ErrMsg.NONE,
            CheckSt.CHECK_ACRONYM, tradeMarkNextStartStrs_, null);
    }
}
