package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import gov.nih.nlm.nls.lvg.Util.UnicodeUtil;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check the citation of a lexRecord.
* <ul>
* <li>citation is chosen from base forms in the following order 
*     <ul>
*     <li>pure ASCII
*   <li>no punctuation
*   <li>shorter length
*   <li>by alphabetic orderoi
*   </ul>
* <li>check glreg ending for noun
* <li>check regd ending for adj and verb
* </ul>
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckCitation
{
    // private constructor
    private CheckCitation()
    {
    }
    // public methods
    // check content itself
    public static boolean CheckContent(LexRecord lexRecord)
    {
        // check extra space
        boolean spaceFlag = CheckContent.CheckSpacesForItem(lexRecord, 
            LexRecordUtil.CONTENT_CIT); 
        boolean validFlag = spaceFlag;
        return validFlag;
    }
    // check current content with other contents in the same record
    public static boolean CheckContents(LexRecord lexRecord)
    {
        // check citation order: nonAScii & punctuation
        boolean orderFlag = CheckOrder(lexRecord);
        boolean glregFlag = CheckGlreg(lexRecord);    // check glreg
        boolean regdFlag = CheckRegd(lexRecord);    // check regd
        boolean validFlag = orderFlag && glregFlag && regdFlag;
        return validFlag;
    }
    // check correct ending if it is verb or adj and regd
    private static boolean CheckRegd(LexRecord lexRecord)
    {
        String base = lexRecord.GetBase();
        boolean validFlag = CheckEntry.CheckRegd(lexRecord, base, 
            LexRecordUtil.CONTENT_CIT);
        return validFlag;
    }
    // check correct ending if it is noun and glreg
    private static boolean CheckGlreg(LexRecord lexRecord)
    {
        String base = lexRecord.GetBase();
        boolean validFlag = CheckEntry.CheckGlreg(lexRecord, base, 
            LexRecordUtil.CONTENT_CIT);
        return validFlag;
    }
    
    // check types: non-ASCII and punctuation citation form
    private static boolean CheckOrder(LexRecord lexRecord)
    {
        boolean validFlag = true;
        int contentType = LexRecordUtil.CONTENT_CIT;
        // reform base forms list from citation and spellVars
        String citation = lexRecord.GetBase();
        Vector<String> spVars = lexRecord.GetSpellingVars();
        Vector<String> bases = new Vector<String>();
        bases.add(citation);
        for(String spVar:spVars)
        {
            bases.add(spVar);
        }
        // sort base forms
        BaseComparator<String> bc = new BaseComparator<String>();
        Collections.sort(bases, bc);
        // check citation
        if(citation.equals(bases.elementAt(0)) == false)
        {
            lexRecord.SetBase(bases.elementAt(0));
            validFlag = false;
        }
        // reset the spelling vars
        bases.removeElementAt(0);    // remove citation form
        lexRecord.SetSpellingVars(bases);    // assign spVars to the sorted order
        // err msg
        if(validFlag == false)
        {
            ErrMsgUtilLexRecord.AddContentErrMsg(contentType, 
                ErrMsgUtilLexRecord.ERR_ORDER, citation, lexRecord);
        }
        return validFlag;
    }
    // data member
}
