package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import gov.nih.nlm.nls.lvg.Util.UnicodeUtil;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
import gov.nih.nlm.nls.lexCheck.Gram.*;
/*****************************************************************************
* This class provides a Java class to check the class types of a lexRecord.
* <ul>
* <li>No duplicates - auto fix option by removing duplicates
* </ul>
*
* <p><b>History:</b>
* <ul>
* <li>SCR-58: chlu, 11/27/18, add a new field class_type - check contents
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckClassTypes
{
    // private constructor
    private CheckClassTypes()
    {
    }
    // public methods
    // check content itself
    public static boolean CheckContent(LexRecord lexRecord)
    {
        // check extra space
        boolean spaceFlag = CheckContent.CheckSpacesForList(lexRecord, 
            LexRecordUtil.CONTENT_CT);
        boolean baseFlag = CheckBase(lexRecord); 
        boolean tbdFlag = CheckTbd(lexRecord); 
        boolean validFlag = spaceFlag && baseFlag && tbdFlag;
        return validFlag;
    }
    // check current content with other contents in the same record
    public static boolean CheckContents(LexRecord lexRecord)
    {
        // check duplicates
        boolean dupFlag = CheckContent.CheckDuplicatesForList(lexRecord, 
            LexRecordUtil.CONTENT_CT);
        boolean validFlag = dupFlag;
        return validFlag;
    }

	// private method
	// this method should be removed once the official release is done with
	// complete tags on all lexical records
	private static boolean CheckTbd(LexRecord lexRecord)
	{
		boolean validFlag = true;
		
		// get class_types
		Vector<String> classTypes = lexRecord.GetClassTypes();
		for(String classType:classTypes)
		{
			if(classType.equals(CheckFormatClassType.CT_TBD) == true)
			{
				// total no for class_type=TBD
				ErrMsgUtilLexRecord.AddClassTypeTbd();

				// illegal class_type=TBD with other class_types
				if(classTypes.size() > 1)
				{
					validFlag = false;
					ErrMsgUtilLexRecord.AddContentErrMsg(
						LexRecordUtil.CONTENT_CT,
						ErrMsgUtilLexRecord.ERR_CT_TBD, 
						classType, lexRecord);
					break;
				}
			}
		}

		return validFlag;
	}
	private static boolean CheckBase(LexRecord lexRecord)
	{
		boolean validFlag = true;
		// get all base forms
		String citation = lexRecord.GetBase();
		Vector<String> svList = lexRecord.GetSpellingVars();
		HashSet<String> baseList = new HashSet<String>(svList);
		baseList.add(citation);

		// get class_types
		Vector<String> classTypes = lexRecord.GetClassTypes();
		for(String classType:classTypes)
		{
			if((classType.startsWith("archaic"))
			|| (classType.startsWith("source"))
			|| (classType.startsWith("informal")))
			{
				String base = classType.split("\\|")[1];
				if(baseList.contains(base) == false)
				{
					validFlag = false;
					String errText = classType + " => " + base;
					ErrMsgUtilLexRecord.AddContentErrMsg(
						LexRecordUtil.CONTENT_CT,
						ErrMsgUtilLexRecord.ERR_CT_BASE, errText, lexRecord);
				}
			}
		}

		return validFlag;
	}
    // data member
}
