package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import gov.nih.nlm.nls.lvg.Util.UnicodeUtil;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check the EUI of a lexRecord.
* <ul>
* <li>Illegal EUI: can't be E0000000
* </ul>
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckEui
{
    // private constructor
    private CheckEui()
    {
    }
    // public methods
    public static boolean CheckContent(LexRecord lexRecord)
    {
        // check non-ASCII citation form
        boolean validFlag = CheckIllegalEui(lexRecord);
        return validFlag;
    }
    // check non-ASCII citation form
    private static boolean CheckIllegalEui(LexRecord lexRecord)
    {
        boolean validFlag = true;
        String eui = lexRecord.GetEui();
        // check if EUI is E0000000
        if(eui.equals("E0000000") == true)
        {
            validFlag = false;
            ErrMsgUtilLexRecord.AddContentErrMsg(LexRecordUtil.CONTENT_EUI, 
                ErrMsgUtilLexRecord.ERR_ILLEGAL, eui, lexRecord);
        }
        return validFlag;
    }
	public static void Init(Vector<LexRecord> lexRecords)
    {
		euiBasePosMap_ = new HashMap<String, BasePosObj>(); 
        // go through all records
        int recSize = lexRecords.size();
        for(int i = 0; i < recSize; i++)
        {
			LexRecord lexRecord = lexRecords.elementAt(i);
			String eui = lexRecord.GetEui();
			BasePosObj basePosObj = new BasePosObj(lexRecord);
			euiBasePosMap_.put(eui, basePosObj);
        }
	}
	// have to call Init first to init euiBasePosMap_
	// get citation by base|eui|cat
	public static String GetCitByEuiBase(String base, String eui, String cat)
    {
        String cit = null;
        if((euiBasePosMap_ == null) || (euiBasePosMap_.size() == 0))
        {
            // err in init
			System.err.println("**ERR@GetCitByEuiBase( ): euiBasePosMap_ is not initiated!");
            return cit;
        }
        else
        {
            BasePosObj basePosObj = euiBasePosMap_.get(eui);
            // no EUI found
            if(basePosObj == null)
            {
                // err
            }
            else if(cat.equals(basePosObj.GetCategory()) == false)
            {
                // err - different cat
			}
            else    // correct cat
			{
				// the input base is the citation
				if(base.equals(basePosObj.GetCit()) == true)
                {
                    cit = base;
                }
                // the input base is a spVar
                else if(basePosObj.GetBases().contains(base) == true)
                {
                    cit = basePosObj.GetCit();
                }
                else
                {
                    //err
				}                    
			}
		}
		return cit;
	}
	// data member
	// EUI|BasePos
	// The first element of BasePosObj is the citation
	// Used in cross-ref check for ABB, ACR, nom and classType EUIs
	// euiBasePosMap_: key:eui, values: basePosObj
	private static HashMap<String, BasePosObj> euiBasePosMap_ = null;
}
