package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import gov.nih.nlm.nls.lvg.Util.UnicodeUtil;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check the Nominalizations of a lexRecord.
* <ul>
* <li>No duplicates - with fix option by removing duplicates
* <li>No "||"
* </ul>
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckNominalizations
{
    // private constructor
    private CheckNominalizations()
    {
    }
    // public methods
    // check content itself
    public static boolean CheckContent(LexRecord lexRecord)
    {
        // check extra space
        boolean spaceFlag = CheckContent.CheckSpacesForList(lexRecord, 
            LexRecordUtil.CONTENT_NOM); 
        // check double pipes with empty data for nom
        boolean dpFlag = CheckContent.CheckDoublePipesForList(lexRecord,
            LexRecordUtil.CONTENT_NOM);    
        // check pipe with category
        boolean catFlag = CheckNomCat(lexRecord); 
        boolean validFlag = spaceFlag && dpFlag && catFlag;
        return validFlag;
    }
    // check current content with other contents in the same record
    public static boolean CheckContents(LexRecord lexRecord)
    {
        // check duplicates
        boolean dupFlag = CheckContent.CheckDuplicatesForList(lexRecord, 
            LexRecordUtil.CONTENT_NOM);
        boolean validFlag = dupFlag;
        return validFlag;
    }
    private static boolean CheckNomCat(LexRecord lexRecord)
    {
        boolean validFlag = true;
        String cat = lexRecord.GetCategory();
        Vector<String> nomList = lexRecord.GetNominalizations();
        
        // check nom category
        for(String nom:nomList)
        {
            if(IsLegalCat(nom, cat) == false)
            {
                validFlag = false;
                ErrMsgUtilLexRecord.AddContentErrMsg(LexRecordUtil.CONTENT_NOM,
                    ErrMsgUtilLexRecord.ERR_ILLEGAL_NOM_CAT, nom, lexRecord);
            }
        }
        return validFlag;
    }
    private static boolean IsLegalCat(String nom, String cat)
    {
        boolean validFlag = true;
        // determine format: nom=citation|cat or nom=citation|cat|EUI
        int index1 = nom.indexOf(LexRecordUtil.FS_STR);
        int index2 = nom.indexOf(LexRecordUtil.FS_STR, index1+1);
        int index3 = nom.indexOf(LexRecordUtil.FS_STR, index2+1);
        String nomCat = null;
        if(index1 == -1)    // error: no "|"
        {
            validFlag = false;
        }
        else if (index2 == -1)    // OK: nom=citation|cat
        {
            nomCat = nom.substring(index1+1);
        }
        else if(index3 == -1)     // OK: nom=citation|cat|EUI
        {
            nomCat = nom.substring(index1+1, index2);
        }
        else // error: more than 3 |||
        {
            validFlag = false;
        }
        // check nomCat
        String noun = LexRecordUtil.GetCategory(LexRecordUtil.CAT_NOUN);
        String adj = LexRecordUtil.GetCategory(LexRecordUtil.CAT_ADJ);
        String verb = LexRecordUtil.GetCategory(LexRecordUtil.CAT_VERB);
        if(nomCat != null)
        {
            if(cat.equals(noun))
            {
                if((nomCat.equals(adj) == false)
                && (nomCat.equals(verb) == false))
                {
                    validFlag = false;
                }
            }
            else if(cat.equals(adj) == true)
            {
                if(nomCat.equals(noun) == false)
                {
                    validFlag = false;
                }
            }
            else if(cat.equals(verb) == true)
            {
                if(nomCat.equals(noun) == false)
                {
                    validFlag = false;
                }
            }
        }
        return validFlag;
    }
    // data member
}
