package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lvg.Util.UnicodeUtil;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check the Variants of a lexRecord.
* <ul>
* <li>No duplicates - with fix option by removing duplicates
* </ul>
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckVariants
{
    // private constructor
    private CheckVariants()
    {
    }
    // public methods
    // check content itself
    public static boolean CheckContent(LexRecord lexRecord)
    {
        // check extra space
        boolean spaceFlag = CheckContent.CheckSpacesForList(lexRecord, 
            LexRecordUtil.CONTENT_VAR); 
        boolean validFlag = spaceFlag;
        return validFlag;
    }
    // check current content with other contents in the same record
    public static boolean CheckContents(LexRecord lexRecord,
        HashSet<String> irregExpEuiList)
    {
        // check duplicates
        boolean dupFlag = CheckContent.CheckDuplicatesForList(lexRecord, 
            LexRecordUtil.CONTENT_VAR);
        boolean irregFlag = CheckIrreg(lexRecord, irregExpEuiList);
        boolean validFlag = dupFlag && irregFlag;
        return validFlag;
    }
    // private method
    // the first field of irreg must be base form
    // all base from must have irreg
    private static boolean CheckIrreg(LexRecord lexRecord, 
        HashSet<String> irregExpEuiList)
    {
        boolean validFlag = true;
        // get irreg variants
        Vector<String> variants = lexRecord.GetVariants();
        // get all base forms
        String citation = lexRecord.GetBase();
        Vector<String> svList = lexRecord.GetSpellingVars();
        HashSet<String> baseList = new HashSet<String>(svList);
        baseList.add(citation);
        // check irreg: if base legal
        HashSet<String> irregBases = new HashSet<String>();
        for(String variant:variants)
        {
            if((variant.startsWith("irreg|"))
            || (variant.startsWith("group(irreg|")))
            {
                // check if legal base
                String irregBase = GetIrregBase(variant); 
                if(baseList.contains(irregBase) == false)
                {
                    validFlag = false;
                    ErrMsgUtilLexRecord.AddContentErrMsg(
                        LexRecordUtil.CONTENT_VAR,
                        ErrMsgUtilLexRecord.ERR_IRREG_BASE, variant, lexRecord);
                }
				else	// add legal irregBase for exceptions
				{
					irregBases.add(irregBase);
				}
            }
        }
		if(validFlag == false)
		{
			return validFlag;
		}
        // check if all bases have irreg, known exceptions are OK
        if(irregBases.size() > 0)	// have legal irreg base
        {
            String eui = lexRecord.GetEui();
			if((baseList.size() != irregBases.size())	// all bases have irreg
            && (irregExpEuiList.contains(eui) == false)) // check if exception
			{
				validFlag = false;
				for(String base:baseList)
				{
					if(irregBases.contains(base) == false)
					{
						ErrMsgUtilLexRecord.AddContentErrMsg(
							LexRecordUtil.CONTENT_VAR,
							ErrMsgUtilLexRecord.ERR_IRREG_MISSING, 
							base, lexRecord);
					}
				}
            }
        }
        return validFlag;
    }
    private static String GetIrregBase(String variant)
    {
        String irregBase = new String();
        int index1 = variant.indexOf("irreg|");
        if(index1 < 0)
        {
            return irregBase;
        }
        int index2 = variant.indexOf("|", index1+7);
        irregBase = variant.substring(index1+6, index2);
        return irregBase;
    }
    public static HashSet<String> GetIrregExpEuiListFromFile(
        String irregExpFile)
    {
        boolean verbose = true;
        int dispLineNum = 100;	// loading status of line number to be displayed
        return GetIrregExpEuiListFromFile(irregExpFile, verbose, dispLineNum);
    }
    public static HashSet<String> GetIrregExpEuiListFromFile(
        String irregExpFile, boolean verbose, int dispLineNum)
    {
        HashSet<String> irregExpEuiList = null;
        boolean useDefaultFileFlag = false;
        // use default irregExceptions from jar file
        if((irregExpFile == null)
        || (irregExpFile.length() == 0)
        || (new File(irregExpFile).exists() == false))
        {
            useDefaultFileFlag = true;
            irregExpFile = "/irregExceptions.data";
        }
        String defaultStr = new String();
        if(useDefaultFileFlag == true)
        {
            defaultStr = " (default file in jar)";     
        }
        System.out.println("===== Get Irreg Exception EUIs from"
            + defaultStr + ": " + irregExpFile);
        irregExpEuiList = SetTable.GetSetTabe(irregExpFile, verbose,
            dispLineNum, useDefaultFileFlag);
        return irregExpEuiList;
    }
    // data member
    private static final String IRREG_STR = "irreg|";
}
