package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check cross-ref EUI on abbreviation.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CrossCheckAbbEui
{
    // private constructor
    private CrossCheckAbbEui()
    {
    }
    // lexRecord is automatically fixed
    public static boolean Check(LexRecord lexRecord,
        HashSet<String> notBaseFormSet) 
    {
        boolean validFlag = true;
        Vector<String> abbList = lexRecord.GetAbbreviations();
        String abbCat = lexRecord.GetCategory();
		// go through all abb list from the input lexRecord
        for(int i = 0; i < abbList.size(); i++)
        {
            String abb = abbList.elementAt(i);
            // tokenize abbreviations
            int index1 = abb.indexOf(LexRecordUtil.FS_STR);
            String abbCit = new String();
            String abbEui  = new String();
            if(index1 > 0) // for abb = citation|Eui
            {
                abbCit = abb.substring(0, index1);
                abbEui = abb.substring(index1+1);
            }
            else    // for abb = citation, no abbEui
            {
                abbCit = abb;
            }
			// init vars
			String citCat = abbCit + LexRecordUtil.FS_STR + abbCat;
			// get citation in Lexicon by Cit, Eui, and cat
			String cit = CheckEui.GetCitByEuiBase(abbCit, abbEui, abbCat);
			// get eui in Lexicon by citation and cat
            HashSet<String> euisByCitCat = 
                CrossCheckDupLexRecords.GetEuisByCitCat(citCat);
			// These checks are better to be proceed by order
			// 2.3. no EUI: the abbEui is not in Lexicon
            if((abbEui.length() > 0)    // has abbEui
            && (euisByCitCat == null))  // no eui found in Lexicon
            {
				// auto-fix: remove EUI
				abbList.set(i, abbCit);
				// sent auto-fix msg
				validFlag = false;
				ErrMsgUtilLexicon.AddContentErrMsg(
					LexRecordUtil.CROSS_CHK_ABB, 
					ErrMsgUtilLexicon.ERR_NO_EUI, 
					abb + " - None", lexRecord);
			}
			else if((abbEui.length() > 0)	// has abbEui
			&& (cit == null) // no cit found by abbCit|abbEui 
			&& (notBaseFormSet.contains(citCat) == false))	// not invalid base
			{
				// auto-fix: remove EUI
				abbList.set(i, abbCit);
				// sent auto-fix msg
				validFlag = false;
				ErrMsgUtilLexicon.AddContentErrMsg(
					LexRecordUtil.CROSS_CHK_ABB,
					ErrMsgUtilLexicon.ERR_NO_EUI,
					abb + " - None", lexRecord);
			}
			// 2.4 & 2.5 wrong citations
			if((abbEui.length() > 0) // has abbEUI in the original file 
			&& (cit != null) // cit found by abbCit|abbEui 
			&& (cit.equals(abbCit) == false)) 
			// abbCit is a base (spVar), not citation
			{
				validFlag = false;
				// auto-fix: change EUI
				String newAbb = cit + LexRecordUtil.FS_STR + abbEui;
				// already exist after changing spVar to citation
				if(abbList.contains(newAbb) == true)
				{
					// auto-fix: remove
					abbList.remove(i);
					// sent auto-fix msg
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ABB,
						ErrMsgUtilLexicon.ERR_WRONG_CIT_SPVAR_DUP,
						abb + " - wrong cit: [" + abbCit
						+ "], is spVar, replaced by cit: [" + cit 
						+ "] becomes duplicates", lexRecord);
				}
				else    // not exist, replace base/spVar by citation
				{
					// auto-fix: use correct citation
					abbList.set(i, newAbb);
					// sent auto-fix msg
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ABB,
						ErrMsgUtilLexicon.ERR_WRONG_CIT_SPVAR,
						abb + " - wrong cit: [" + abbCit
						+ "], is spVar, replaced by cit: [" + cit + "]",
						lexRecord);
				}
			}
			// single EUI: 2.6 & 2.7 missing EUI & wrong EUI
            if((euisByCitCat != null)
			&& (euisByCitCat.size() == 1))    // single EUI in Lexicon
            {
                Vector<String> euiList = new Vector<String>(euisByCitCat);
                String newEui = euiList.elementAt(0);
                if(abbEui.length() > 0)		// has abbEui
                {
                    if(euisByCitCat.contains(abbEui) == true)
                    {
                        // One EUI match, do nothing, correct
                    }
                    else	// no EUI match
                    {
						// no auto-fix
                        // sent warning msg
                        validFlag = false;
                        ErrMsgUtilLexicon.AddContentErrMsg(
                            LexRecordUtil.CROSS_CHK_ABB,
                            ErrMsgUtilLexicon.ERR_WRONG_EUI, 
                            abb + " - " + newEui, lexRecord);
                    }
                }
                else    // no abbEui => Add it in
                {
                    // auto-fix
                    String newAbb = abb + LexRecordUtil.FS_STR + newEui;
                    abbList.set(i, newAbb);
                    // sent out auto fix msg
                    validFlag = false;
                    ErrMsgUtilLexicon.AddContentErrMsg(
                        LexRecordUtil.CROSS_CHK_ABB,
                        ErrMsgUtilLexicon.ERR_MISS_EUI, 
                        abb + " - " + newEui, lexRecord);
                }
            }
			// multiple EUIs: 2.8 & 2.9 missing EUIs & wrong EUIs
            if((euisByCitCat != null)
			&& (euisByCitCat.size() > 1)) // multiple EUIs in Lexicon
            {
                Vector<String> euiList = new Vector<String>(euisByCitCat);
                if(abbEui.length() > 0)
                {
                    if(euisByCitCat.contains(abbEui) == true)
                    {
                        // One EUI match, do nothing, correct
                    }
                    else    // no eui even multiple matched EUI found
                    {
						// no auto-fix
                        // sent warning msg
                        validFlag = false;
                        ErrMsgUtilLexicon.AddContentErrMsg(
                            LexRecordUtil.CROSS_CHK_ABB,
                            ErrMsgUtilLexicon.ERR_WRONG_EUIS, 
                            abb + " - " + euiList, lexRecord);
                    }
                }
                else    // no abbEui
                {
					// no auto-fix
                    // sent out msg
                    validFlag = false;
                    ErrMsgUtilLexicon.AddContentErrMsg(
                        LexRecordUtil.CROSS_CHK_ABB,
                        ErrMsgUtilLexicon.ERR_MISS_EUIS, 
                        abb + " - " + euiList, lexRecord);
                }
            }
			// 2.13 new EUI for candidates
			if((euisByCitCat == null)  // no eui in Lexicon
			&& (abbEui.length() == 0))	// no abbEui
            {
				// no EUI matches: send msg to add to to-do list
				// send errMsg only if expansion is not one of notBaseForm
				if(notBaseFormSet.contains(citCat) == false)
				{
					validFlag = false;
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ABB,
						ErrMsgUtilLexicon.ERR_NEW_EUI, 
						abb + " - New", lexRecord);
				}
            }
			else if(abbEui.length() == 0) // no abbEui
			{
				// this is not suppose to happen if all miss EUi are fixed
				// new citation without EUI, could be used for new record
				if(notBaseFormSet.contains(citCat) == false)
                {
                    validFlag = false;
                    ErrMsgUtilLexicon.AddContentErrMsg(
                        LexRecordUtil.CROSS_CHK_ABB,
                        ErrMsgUtilLexicon.ERR_NEW_EUI,
                        abb + " - New2", lexRecord);
                }
			}
        }
        return validFlag;
    }
    // data member
}
