package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check cross-ref EUI on acronyms.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CrossCheckAcrEui
{
    // private constructor
    private CrossCheckAcrEui()
    {
    }
    // lexRecord is automatically fixed
    public static boolean Check(LexRecord lexRecord, 
        HashSet<String> notBaseFormSet) 
    {
        boolean validFlag = true;
        Vector<String> acrList = lexRecord.GetAcronyms();    // expansion
        String acrCat = lexRecord.GetCategory();
		// go through all acr list from the input lexRecord
        for(int i = 0; i < acrList.size(); i++)
        {
            String acr = acrList.elementAt(i);
            // tokenize acronym
            int index1 = acr.indexOf(LexRecordUtil.FS_STR);
            String acrCit = new String();    // expansion
            String acrEui  = new String();	// EUI
            if(index1 > 0) // for acr = citation|Eui
            {
                acrCit = acr.substring(0, index1);
                acrEui = acr.substring(index1+1);
            }
            else    // for acr = citation
            {
                acrCit = acr;
            }
			//init vars
			String citCat = acrCit + LexRecordUtil.FS_STR + acrCat;
			// get citation in Lexicon by Cit, Eui, and cat
			String cit = CheckEui.GetCitByEuiBase(acrCit, acrEui, acrCat);
			// get eui in Lexicon by citation and cat
            HashSet<String> euisByCitCat = 
                CrossCheckDupLexRecords.GetEuisByCitCat(citCat);
			// These checks are better to be proceeded by order
			// 2.3. no EUI: the acrEui is not in Lexicon
            if((acrEui.length() > 0)    // has acrEui
            && (euisByCitCat == null))    // no eui in Lexicon
			{
				// auto-fix
				acrList.set(i, acrCit);
				// sent auto-fix msg
				validFlag = false;
				ErrMsgUtilLexicon.AddContentErrMsg(
					LexRecordUtil.CROSS_CHK_ACR,
					ErrMsgUtilLexicon.ERR_NO_EUI, 
					acr + " - None", lexRecord);
			}
			else if((acrEui.length() > 0)   // has acrEui
			&& (cit == null) // no cit found by abbCit|abbEui 
			&& (notBaseFormSet.contains(citCat) == false)) // not invalid base
			{
				// auto-fix: remove EUI
				acrList.set(i, acrCit);
				// sent auto-fix msg
				validFlag = false;
				ErrMsgUtilLexicon.AddContentErrMsg(
					LexRecordUtil.CROSS_CHK_ACR, 
					ErrMsgUtilLexicon.ERR_NO_EUI, 
					acr + " - None", lexRecord);
			}
		
			// 2.4 & 2.5 wrong citations
			if((acrEui.length() > 0) // has acr|EUI in the original file
			&& (cit != null) // cit found by acrCit|acrEui 
			&& (cit.equals(acrCit) == false))
			// acrCit is a base (spVar), not citation
			{
				validFlag = false;
				// auto-fix: change cit
				String newAcr = cit + LexRecordUtil.FS_STR + acrEui;
				// already exist after changing spVar to citation
				if(acrList.contains(newAcr) == true)
				{
					// auto-fix: remove
					acrList.remove(i);
					// sent auto-fix msg
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ACR,
						ErrMsgUtilLexicon.ERR_WRONG_CIT_SPVAR_DUP,
						acr + " - wrong cit: [" + acrCit
						+ "], is spVar, replaced by cit: [" + cit
						+ "] becomes duplicates", lexRecord);
				}
				else	// not exist, replace base/spVar by citation
				{
					// auto-fix: use correct citation
					acrList.set(i, newAcr);
					// sent auto-fix msg
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ACR, 
						ErrMsgUtilLexicon.ERR_WRONG_CIT_SPVAR, 
						acr + " - wrong cit: [" + acrCit
						+ "], is spVar, replaced by cit: [" + cit + "]",
						lexRecord);
				}
			}
			// single EUI: 2.6 & 2.7 missing EUI & wrong EUI
			if((euisByCitCat != null)
            && (euisByCitCat.size() == 1))    // single EUI in Lexicon
            {
                Vector<String> euiList = new Vector<String>(euisByCitCat);
                String newEui = euiList.elementAt(0);
                if(acrEui.length() > 0)
                {
                    if(euisByCitCat.contains(acrEui) == true)
                    {
                        // One EUI match, do nothing, correct
                    }
                    else    // wrong EUI
                    {
                        // sent warning msg
                        validFlag = false;
                        ErrMsgUtilLexicon.AddContentErrMsg(
                            LexRecordUtil.CROSS_CHK_ACR,
                            ErrMsgUtilLexicon.ERR_WRONG_EUI, 
                            acr + " - " + newEui, lexRecord);
                    }
                }
                else    // no acrEui => Add it in
                {
                    // auto-fix
                    String newAcr = acr + LexRecordUtil.FS_STR + newEui;
                    acrList.set(i, newAcr);
                    // sent out auto fix msg
                    validFlag = false;
                    ErrMsgUtilLexicon.AddContentErrMsg(
                        LexRecordUtil.CROSS_CHK_ACR,
                        ErrMsgUtilLexicon.ERR_MISS_EUI, 
                        acr + " - " + newEui, lexRecord);
                }
            }
			// multiple EUIs: 2.8 & 2.9 missing EUIs & wrong EUIs
			if((euisByCitCat != null)
            && (euisByCitCat.size() > 1))    // multiple EUIs in Lexicon
            {
                Vector<String> euiList = new Vector<String>(euisByCitCat);
                if(acrEui.length() > 0)
                {
                    if(euisByCitCat.contains(acrEui) == true)
                    {
                        // One EUI match, do nothing, correct
                    }
                    else    // no eui even multiple matched EUI found
                    {
                        // sent warning msg
                        validFlag = false;
                        ErrMsgUtilLexicon.AddContentErrMsg(
                            LexRecordUtil.CROSS_CHK_ACR,
                            ErrMsgUtilLexicon.ERR_WRONG_EUIS, 
                            acr + " - " + euiList, lexRecord);
                    }
                }
                else    // no acrEui
                {
                    // sent out msg
                    validFlag = false;
                    ErrMsgUtilLexicon.AddContentErrMsg(
                        LexRecordUtil.CROSS_CHK_ACR,
                        ErrMsgUtilLexicon.ERR_MISS_EUIS, 
                        acr + " - " + euiList, lexRecord);
                }
			}
					
			// 2.13 new EUI for candidates
            if((euisByCitCat == null)    // no eui in Lexicon
			&& (acrEui.length() == 0))  // no acrEui
            {
				// no EUI matches: send msg to add to to-do list
				// send errMsg only if expansion is not one of notBaseForm
				if(notBaseFormSet.contains(citCat) == false)
				{
					validFlag = false;
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ACR,
						ErrMsgUtilLexicon.ERR_NEW_EUI, 
						acr + " - New", lexRecord);
				}
            }
			else if(acrEui.length() == 0)	// no acrEui
			{
				// this is not suppose to happen if all miss EUI are fixed
				// new citation without EUI, could be used for new record
                if(notBaseFormSet.contains(citCat) == false)
                {
                    validFlag = false;
				    ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ACR,
						ErrMsgUtilLexicon.ERR_NEW_EUI, 
						acr + " - New2", lexRecord);
				}
			}
        }
        return validFlag;
    }
    // data member
}
