package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check all possible duplicated lexRecord.
* First, load key (citation|cat) - value (EUI) into a hashtable.
* Second, log all duplciated key (possible duplicated record.
* Third, use this hashtable for eui|citation check.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CrossCheckDupLexRecords
{
    // private constructor
    private CrossCheckDupLexRecords()
    {
    }
    // lexRecord is automatically fixed
    public static boolean Check(Vector<LexRecord> lexRecords, 
        Hashtable<String, HashSet<String>> dupRecExpList, 
        BufferedWriter out, boolean verbose) throws IOException
    {
        boolean validFlag = true;
        // load all record to tables
        int recSize = lexRecords.size();
        for(int i = 0; i < recSize; i++)
        {
            LexRecord lexRecord = lexRecords.elementAt(i);
            String cit = lexRecord.GetBase();
            String cat = lexRecord.GetCategory();
            String eui = lexRecord.GetEui();
            // citCat|Euis
            String citCat = cit + LexRecordUtil.FS_STR + cat;    // key
            boolean dupFlag = AddToCitCatEuisTable(citCat, eui);
            validFlag = validFlag && dupFlag;
            /**
            // baseCat|Euis
            Vector<String> spVars = lexRecord.GetSpellingVars();
            for(String spVar:spVars)
            {
                String baseCat = spVar + LexRecordUtil.FS_STR + cat;   // key
                AddToBaseCatEuisTable(baseCat, eui);
            }
            // Eui|citation
            AddToEuiCitTable(eui, cit);
            **/
            if(verbose == true)
            {
                if((i%100000 == 0) && (i > 0))
                {
                    System.out.println("- Loaded " + i 
                        + " lexRecords to table");
                }
            }
        }
        if(verbose == true)
        {
            System.out.println("- Complete loaded " + recSize 
                + " lexRecords to hash table for dupRec check");
        }
        // Print out potential duplicated records
        if(validFlag == false)
        {
            // check citCatEuisTable_ for duplicated lexRecord
            Enumeration<String> keys = citCatEuisTable_.keys();
            while(keys.hasMoreElements() == true)
            {
                String key = keys.nextElement();    // citation|cat
                HashSet<String> euis = citCatEuisTable_.get(key);
                boolean dupFlag = false;
                // euis has same citation|cat are possible duplicated lexRecord
                if(euis.size() > 1)
                {
                    Iterator<String> it = euis.iterator();
                    while(it.hasNext() == true)
                    {
                        // check if it is an exception
                        String eui = it.next();
                        HashSet<String> dupRecExpEuis = dupRecExpList.get(key);
                        if((dupRecExpEuis == null) // no execptions
                        || (dupRecExpEuis.contains(eui) == false)) //not exp
                        {
                            dupFlag = true;
                        }
                    }
                    // print out all possible duplicates
                    if(dupFlag == true)
                    {
                        String errMsg = key;
                        Iterator<String> it2 = euis.iterator();
                        while(it2.hasNext() == true)
                        {
                            errMsg += LexRecordUtil.FS_STR + it2.next();
                        }
                        out.write(errMsg + "|");
                        out.newLine();
                        ErrMsgUtilLexicon.AddContentErrMsg(
                            LexRecordUtil.CROSS_CHK_DUP_REC,
                            ErrMsgUtilLexicon.ERR_DUP_REC, errMsg);
                    }
                }
            }
        }
    
        return validFlag;
    }
    public static HashSet<String> GetEuisByCitCat(String citCat)
    {
        return citCatEuisTable_.get(citCat);
    }
    public static HashSet<String> GetEuisByBaseCat(String baseCat)
    {
        return baseCatEuisTable_.get(baseCat);
    }
    public static String GetCitByEui(String citation)
    {
        return euiCitTable_.get(citation);
    }
    // private method
    private static boolean AddToCitCatEuisTable(String key, String value) 
    {
        boolean validFlag = true;
        // add a new values if key did not exist
        if(citCatEuisTable_.containsKey(key) == false)
        {
            HashSet<String> values = new HashSet<String>();
            values.add(value);
            citCatEuisTable_.put(key, values);
        }
        else    // add value to values if key exist
        {
            // add value to the values with same key
            if(citCatEuisTable_.get(key).add(value) != true)
            {
                System.err.println(
                    "** Err@CrossCheckDupLexRecords.AddToCitCatEuisTable[" 
                    + key + ", " + value + "]");
            }
            // potential duplicated records with exception list
            // same citation|category
            validFlag = false;
        }
        return validFlag;
    }
    private static void AddToBaseCatEuisTable(String key, String value) 
    {
        // add a new values if key did not exist
        if(baseCatEuisTable_.containsKey(key) == false)
        {
            HashSet<String> values = new HashSet<String>();
            values.add(value);
            baseCatEuisTable_.put(key, values);
        }
        else    // add value to values if key exist
        {
            // add value to the values with same key
            if(baseCatEuisTable_.get(key).add(value) != true)
            {
                // value exist in the set
                System.err.println(
                    "** Err@CrossCheckDupLexRecords.AddToBaseCatEuisTable[" 
                    + key + ", " + value + "]");
            }
        }
    }
    private static void AddToEuiCitTable(String key, String value) 
    {
        // add a new values if key did not exist
        if(euiCitTable_.containsKey(key) == false)
        {
            euiCitTable_.put(key, value);
        }
        else    // add value to values if key exist
        {
            // value exist in the set
            System.err.println(
                "** Err@CrossCheckDupLexRecords.AddToEuiCitTable[" 
                + key + ", " + value + "]");
        }
    }
    // data member
    // citCat|Euis
    private static Hashtable<String, HashSet<String>> citCatEuisTable_
        = new Hashtable<String, HashSet<String>>();
    // baseCat|Euis
    private static Hashtable<String, HashSet<String>> baseCatEuisTable_
        = new Hashtable<String, HashSet<String>>();
    // Eui|cit    
    private static Hashtable<String, String> euiCitTable_
        = new Hashtable<String, String>();
}
