package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.io.*;
import java.util.*; 
import gov.nih.nlm.nls.lexCheck.Lib.*;
/*****************************************************************************
* This program is to used to load Strings to a Set table from a file.
* The table is stored in a hash set as HashSet&lt;String&gt; and can be used
* for further retrieval.  It also eliminates duplicates.
* 
* File Format: String line
*
* <p><b>History:</b>
*
* @author NLM NLS Development Team
*
* @version    V-2019
****************************************************************************/
public class SetTable
{
    // private constructor
    private SetTable()
    {
    }
    // public methods
    /**
    * Check if a term is in the hash table
    *
    * @param inStr the term to be checked
    * @param setTable a set table stores all terms
    *
    * @return true if the term is in hash table, false if otherwise.
    */
    public static boolean InTable(String inStr, HashSet<String> setTable)
    {
        return setTable.contains(inStr);
    }
    /**
    * Get the set table by loading all terms from a file with options of 
    * verbose and show loading lineNum.
    *
    * @param inFile the file includes terms to be loaded
    * @param verbose verbose flag
    * @param lineNum loading status of line number to be displayed
	* @param useDefaultFileFlag  boolean flag for using default file
    *
    * @return the set table of all terms
    */
    public static HashSet<String> GetSetTabe(String inFile, boolean verbose, 
        int lineNum, boolean useDefaultFileFlag)
    {
        String line = null;
        int lineNo = 0;
        int dupNo = 0;
        HashSet<String> setTable = new HashSet<String>();
        Vector<String> duplicates = new Vector<String>();
        try
        {
            BufferedReader in = null;
            // use default file from jar
            if(useDefaultFileFlag == true)
            {
                in = new BufferedReader(new InputStreamReader(
                    SetTable.class.getResourceAsStream(inFile)));
            }
            else    // use specified file
            {
                in = new BufferedReader(new InputStreamReader(
                    new FileInputStream(inFile), "UTF-8"));
            }
            // read in line by line from a file
            while((line = in.readLine()) != null)
            {
                // skip the line if it is empty or comments (#)
                if((line.length() > 0) && (line.charAt(0) != '#'))
                {
                    // check duplicates & add to the tabl
                    if(setTable.add(line) == false)
                    {
                        duplicates.add(line);
                        dupNo++;
                    }
                    lineNo++;
                }
                if((verbose == true)
                && (lineNo%lineNum == 0)
                && (lineNo != 0))
                {
                    System.out.println("-- current line: " + lineNo);
                }
            }
            // print out message
            if(verbose == true)
            {
                System.out.println("-----------------------------");
                System.out.println("--- Total line: " + lineNo);
                System.out.println("--- Total duplicates: " + dupNo);
                System.out.println("------ Duplicates -------");
                System.out.println(VectorToString(duplicates));
                System.out.println("-------------------------");
            }
            // close files
            in.close();
        }
        catch(Exception e)
        {
            System.err.println("** ERR@SetTable.GetSetTabe (" + inFile + ")");
            System.err.println("** ERR@(" + lineNo + "): " + line);
            System.err.println("Exception: " + e.toString());
        }
        return setTable;
    }
    // Convert a collection of Strings to a string.
    public static String VectorToString(Vector<String> inList)
    {
        String outStr = new String();
        for(String inItem:inList)
        {
            outStr += inItem + GlobalVars.LS_STR;
        }
        return outStr;
    }
    
    // data members
}
