package gov.nih.nlm.nls.lexCheck.Compl;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import java.io.*;
import java.util.*;
/*****************************************************************************
* This class checks if the input term is a legit adverbiral particle in lexicon.
* Use the particle file described in the configuration file.
* The particle file is defined in the GlobalVars.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-48, chlu, 12-11-18, check particles in verb particle constructions
* </ul>
*
* @author NLM Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckParticle
{
    // public constructor
    public CheckParticle()
    {
        // read prepostion from file
        String inFile = GlobalVars.GetParticleFile();
        Init(inFile);
    }
    // public method
    /**
    * Check if the input term is a legit particle in lexicon
    *
    * @param   term  the filler term for checking
    *
    * @return  true or false for a legal or illegal particle
    */
    public static boolean IsLegal(String term) 
    {
        // init particle_ list
        if(particle_.size() == 0)
        {
            // get particle file from config file in global vars
            String particleFile = GlobalVars.GetParticleFile();
            Init(particleFile);
        }
        boolean flag = particle_.contains(term);
        return flag;
    }
    // private method
    private static void Init(String inFile)
    {
        // read particle from input file
        String line = null;
        int partNo = 0;
        try
        {
            BufferedReader in = null;
            // get particle from a file
            if((inFile.length() != 0)
            && (new File(inFile).exists() == true))
            {
                in = new BufferedReader(new InputStreamReader(
                    new FileInputStream(inFile), "UTF-8"));
            }
            else    // get particle from jar file (api.jar or dist.jar)
            {
                inFile = "/particles.data";
                in = new BufferedReader(new InputStreamReader(
                    CheckParticle.class.getResourceAsStream(inFile)));
            }
            GlobalVars.SetParticleFile(inFile);
            // read in line by line from a file
            while((line = in.readLine()) != null)
            {
                // skip the line if it is empty or comments (#)
                if((line.length() > 0) && (line.charAt(0) != '#'))
                {
                    partNo++;
                    particle_.add(line);
                }
            }
            in.close();
        }
        catch(Exception e)
        {
            System.err.println(
				"** Err@CheckParticle.Init(): problem of reading file: "
                + inFile + " at line: " + partNo);
            System.err.println("Exception: " + e.toString());
        }
		// error msg
		if(partNo == 0)
		{
			System.err.println(
				"** ERR@CheckParticle.Init(): no particle (" + partNo 
				+ ") read in from file [" + inFile + "]"); 
		}
    }
    // data member
    private static final int PARTICLE_NUM = 50;
    private static HashSet<String> particle_ 
        = new HashSet<String>(PARTICLE_NUM);
}
