package gov.nih.nlm.nls.lexCheck.Compl;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import java.io.*;
import java.util.*;
/*****************************************************************************
* This class checks if the input term is a legal preposition in lexicon.
* Use the preposition file described in the configuration file.
* The preposition file is defined in the GlobalVars
*
* <p><b>History:</b>
*
* @author NLM Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckPreposition
{
    // public constructor
    public CheckPreposition()
    {
        // read prepostion from file
        String inFile = GlobalVars.GetPrepositionFile();
        Init(inFile);
    }
    // public method
    /**
    * Check if the input term is a legal preposition in lexicon
    *
    * @param   term  the filler term for checking
    *
    * @return  true or false for a legal or illegal preposition
    */
    public static boolean IsLegal(String term) 
    {
        // init preposition_ list
        if(preposition_.size() == 0)
        {
            // get preposition file from config file in global vars
            String prepositionFile = GlobalVars.GetPrepositionFile();
            Init(prepositionFile);
        }
        boolean flag = preposition_.contains(term);
        return flag;
    }
    // private method
    private static void Init(String inFile)
    {
        // read prepositin from input file
        String line = null;
        int prepNo = 0;
        try
        {
            BufferedReader in = null;
            // get preposition from a specified file
            if((inFile.length() != 0)
            && (new File(inFile).exists() == true))
            {
                in = new BufferedReader(new InputStreamReader(
                    new FileInputStream(inFile), "UTF-8"));
            }
            else    // get prepostion from jar file (api.jar or dist.jar)
            {
                inFile = "/prepositions.data";
                in = new BufferedReader(new InputStreamReader(
                    CheckPreposition.class.getResourceAsStream(inFile)));
            }
            GlobalVars.SetPrepositionFile(inFile);
            // read in line by line from a file
            while((line = in.readLine()) != null)
            {
                // skip the line if it is empty or comments (#)
                if((line.length() > 0) && (line.charAt(0) != '#'))
                {
                    prepNo++;
                    preposition_.add(line);
                }
            }
            in.close();
        }
        catch(Exception e)
        {
            System.err.println(
				"** Err@CheckPreposition.Init(): problem of reading file: "
                + inFile + " at line: " + prepNo);
            System.err.println("Exception: " + e.toString());
        }
        // preposition not included in the lexicon: adj
        //preposition_.add("up with"); => delete, not in the recprd
        // preposition not included in the lexicon: verb
        //preposition_.add("together"); => delete, not in the recprd
        //preposition_.add("aside"); => delete, not in the recprd
        //preposition_.add("away"); => delete, not in the recprd
        //preposition_.add("other"); => delete, not in the recprd
        //preposition_.add("accross"); => typo of across
        //preposition_.add("pf"); => typo of of

		// error msg
		if(prepNo == 0)
		{
			System.err.println(
				"** ERR@CheckPreposition.Init(): no preposition (" + prepNo
				+ ") read in from file [" + inFile + "]");
		}
    }
    // data member
    private static final int PREPOSITION_NUM = 200;
    private static HashSet<String> preposition_ 
        = new HashSet<String>(PREPOSITION_NUM);
}
