package gov.nih.nlm.nls.lexCheck.Gram;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
/*****************************************************************************
* This class checks if the filler is a legal class_type format.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-58: chlu, 11/19/18, add a new field class_type - check syntax
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckFormatClassType implements CheckFormat
{
    // public method
    /**
    * Check if the filler is a legal class_type format
    *
    * @param   filler  the input filler for checking
    
    * @return  true or false for a legal or illegal format of class_type
    */
    public boolean IsLegalFormat(String filler)
    {
        // blank class_type, must have a filler
        if((filler == null) || (filler.length() == 0))
        {
            return false;
        }

		// check format of filler
		// split handles "||" if it is not at the end of the string.
		// Split ignores all "\\" if it is at the end of the string.
		String[] args = filler.split("\\|");

		if(filler.endsWith("|") == true)
		{
			return false;
		}

		// filler has only one field
		if(args.length <= 1)		// only one agrument, no field separator
		{
			// class_type=other, class_types=TBD
			if((filler.equals(CT_OTHER) == true)
			|| (filler.equals(CT_TBD) == true))	//removed once tags completed
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		else	// have 2+ fields
		{
			String arg0 = args[0];	// class type
			// class_type = archaic|arg1
			if(arg0.equals(CT_ARCHAIC) == true)
			{
				if(args.length == 2)
				{
					String arg1 = args[1];
					// base form is checked in the checkContent
					if(arg1.length() > 0)	// base
					{
						return true;
					}
					else
					{
						return false;	// illegal source type
					}
				}
				else
				{
					return false;	// extra fields
				}
			}
			// class_type = source|arg1|arg2
			else if(arg0.equals(CT_SOURCE) == true)
			{
				if(args.length == 3)
				{
					String arg1 = args[1];
					String arg2 = args[2];
					if((arg1.length() > 0)	// base
					&& (sourceTypes_.contains(arg2) == true))	// source
					{
						return true;
					}
					else
					{
						return false;	// illegal source type
					}
				}
				else
				{
					return false;	// extra fields
				}
			}
			// calss_type = informal|arg1|arg2|arg3
			// arg1: base form, arg2: formal expression, arg3: EUI
			else if(arg0.equals(CT_INFORMAL) == true)
			{
				if(args.length == 3)
				{
					String arg1 = args[1];
					String arg2 = args[2];
					if((arg1.length() > 0)	// base
					&& (arg2.length() > 0))	// formal expression
					{
						return true;
					}
					else
					{
						return false;	// illegal source type
					}
				}
				else if(args.length == 4)
				{
					String arg1 = args[1];
					String arg2 = args[2];
					String arg3 = args[3];
					if((arg1.length() > 0)	// base
					&& (arg2.length() > 0)	// formal expression
					&& (checkFormatEui_.IsLegalFormat(arg3)))	// EUi format
					{
						return true;
					}
					else
					{
						return false;	// illegal source type
					}
				}
				else
				{
					return false;	// less or extra fields
				}
			}
			else
			{
				return false;
			}
		}
    }
	// data member
    private static CheckFormatEui checkFormatEui_ = new CheckFormatEui();

	public final static String CT_OTHER = "other";
	public final static String CT_TBD = "TBD";
	public final static String CT_ARCHAIC = "archaic";
	public final static String CT_SOURCE = "source";
	public final static String CT_INFORMAL = "informal";
	public static HashSet<String> sourceTypes_ = new HashSet<String>();

	static
	{
		sourceTypes_.add("british");	// British
		sourceTypes_.add("canadian");	// Canadian
		sourceTypes_.add("french");		// French
		sourceTypes_.add("german");		// German
		sourceTypes_.add("greek");		// Greek
		sourceTypes_.add("italian");	// Italian
		sourceTypes_.add("latin");		// Latin
		sourceTypes_.add("other");		// Other
		sourceTypes_.add("spanish");	// Spanish
	}
}
