package gov.nih.nlm.nls.lexCheck.Gram;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.CkLib.*;
import gov.nih.nlm.nls.lexCheck.Cat.*;
/****************************************************************************
* This class provides methods of Grammer check for LexCheck, top level.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-58: chlu, 11/16/18, add a new field class_type - check syntax
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckGrammer
{
    // public method
    /**
    * Check if the grammer syntax for the input lexical record
    *
    * @param   lineObject  the java object of the input line
    * @param   printFlag   the flag for print out check result
    * @param   st       the state of current checking
    * @param   catSt       the state of category checking
    * @param   debugFlag   the flag for print out debug information
    *
    * @return  true or false for a legal or illegal syntax
    */
    public static boolean Check(LineObject lineObject, boolean printFlag,
        CheckSt st, CheckSt catSt, boolean debugFlag)
    {
        boolean flag = true;
        // check each line by line

        switch(st.GetCurState())
        {
            case CheckSt.CHECK_BASE:    // {base=
                lexRecord_ = new LexRecord();        // reset lexRecord
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_,
                    checkBase_, new UpdateBase(), CheckCode.FQ_ONE, true);
                PrintStep(CheckSt.CHECK_BASE, debugFlag, lineObject.GetLine());
                break;
            case CheckSt.CHECK_SPELLING_VAR:    // spelling_variant=
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_, 
                    checkSpellingVar_, new UpdateSpellingVar(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_SPELLING_VAR, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_ENTRY:    // entry=
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_,
                    checkEntry_, new UpdateEntry(), CheckCode.FQ_ONE, true);
                PrintStep(CheckSt.CHECK_ENTRY, debugFlag, lineObject.GetLine());
                break;
            case CheckSt.CHECK_CAT:    // cat=
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_,
                    checkCat_, new UpdateCat(), CheckCode.FQ_ONE, true);
                PrintStep(CheckSt.CHECK_CAT, debugFlag, lineObject.GetLine());
                break;
            case CheckSt.CHECK_CAT_ENTRY:    // slot=filler    
                flag = CheckCatEntry.Check(lineObject, printFlag, st, catSt,
                    lexRecord_, CheckSt.CHECK_ACRONYM, debugFlag);
                PrintStep(CheckSt.CHECK_CAT_ENTRY, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_ACRONYM:    // acronym_of=filler
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_,
                    checkAcronym_, new UpdateAcronym(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_ACRONYM, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_ABBREVIATION:    // abbreviation_of=filler
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_,
                    checkAbbreviation_, new UpdateAbbreviation(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_ABBREVIATION, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_CLASS_TYPE:    // class_type=
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_, 
                    checkClassType_, new UpdateClassType(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_CLASS_TYPE, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_ANNOTATION:    // annotation=filler
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_,
                    checkAnnotation_, new UpdateAnnotation(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_ANNOTATION, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_SIGNATURE:    // signature=filler
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_,
                    checkSignature_, new UpdateSignature(), 
                    CheckCode.FQ_ZERO_ONE, true);
                PrintStep(CheckSt.CHECK_SIGNATURE, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_END:    // }
                flag = CheckCode.Check(lineObject, printFlag, st, lexRecord_,
                    checkEnd_, new UpdateEnd(), CheckCode.FQ_ONE_WHOLE_LINE,
                    false);
                PrintStep(CheckSt.CHECK_END, debugFlag, lineObject.GetLine());
                break;
            default:
                //System.out.println(line);
                break;
        }
        return flag;
    }
    /**
    * Get the checked lexical record
    *
    * @return  the checked lexical record in Java object of LexRecord
    */
    public static LexRecord GetLexRecord()
    {
        return lexRecord_;
    }
    // private methods
    private static void PrintStep(int state, boolean debugFlag, String line)
    {
        if(debugFlag == true)
        {
            switch(state)
            {
                case CheckSt.CHECK_BASE:
                    System.out.println("-- Checked Base: '" + line + "'");
                    break;
                case CheckSt.CHECK_SPELLING_VAR:
                    System.out.println("-- Checked SPELLING_VAR: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_ENTRY:    // entry=
                    System.out.println("-- Checked Entry: '" + line + "'");
                    break;
                case CheckSt.CHECK_CAT:
                    System.out.println("-- Checked Cat: '" + line + "'");
                    break;
                case CheckSt.CHECK_CAT_ENTRY:
                    System.out.println("-- Checked Cat Entry: '" + line + "'");
                    break;
                case CheckSt.CHECK_ACRONYM:
                    System.out.println("-- Checked Acronym: '" + line + "'");
                    break;
                case CheckSt.CHECK_ABBREVIATION:
                    System.out.println("-- Checked Abbreviation: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_CLASS_TYPE:
                    System.out.println("-- Checked CLASS_TYPE: '" + line + "'");
                    break;
                case CheckSt.CHECK_ANNOTATION:
                    System.out.println("-- Checked Annotation: '" + line + "'");
                    break;
                case CheckSt.CHECK_SIGNATURE:
                    System.out.println("-- Checked Signature: '" + line + "'");
                    break;
                case CheckSt.CHECK_END:
                    System.out.println("-- Checked End: '" + line + "'");
                    break;
                default:
                    break;
            }
        }
    }
    // data member
    private static LexRecord lexRecord_ = new LexRecord();
    private static CheckObject checkBase_ = null;
    private static CheckObject checkSpellingVar_ = null;
	// possible next slot names
    private static HashSet<String> spellingVarNextStartStrs_ 
        = new HashSet<String>(2);
    private static CheckObject checkEntry_ = null;
    private static CheckObject checkCat_ = null;
    private static CheckObject checkAcronym_ = null;
    private static HashSet<String> acronymNextStartStrs_ 
        = new HashSet<String>(4);
    private static CheckObject checkAbbreviation_ = null;
    private static HashSet<String> abbreviationNextStartStrs_ 
        = new HashSet<String>(3);
    private static CheckObject checkClassType_ = null;
    private static HashSet<String> classTypeNextStartStrs_ 
        = new HashSet<String>(4);
    private static CheckObject checkAnnotation_ = null;
    private static HashSet<String> annotationNextStartStrs_ 
        = new HashSet<String>(2);
    private static CheckObject checkSignature_ = null;
    private static HashSet<String> signatureNextStartStrs_ 
        = new HashSet<String>(1);
    private static CheckObject checkEnd_ = null;
    static
    {
        // slot base=
        checkBase_ = new CheckObject("{base=", ErrMsg.BASE_SLOT_ERR, 
            ErrMsg.NONE, CheckSt.CHECK_SPELLING_VAR, null, null);
        // slot spelling_variant=
        spellingVarNextStartStrs_.add("entry");
        checkSpellingVar_ = new CheckObject("spelling_variant=",
            ErrMsg.SPELLING_VAR_SLOT_ERR, ErrMsg.NONE,
            CheckSt.CHECK_ENTRY, spellingVarNextStartStrs_, null);
        // slot entry=
        checkEntry_ = new CheckObject("entry=", ErrMsg.ENTRY_SLOT_ERR, 
            ErrMsg.ENTRY_FILLER_ERR, CheckSt.CHECK_CAT, null, 
            new CheckFormatEui());
        // slot     cat=
        checkCat_ = new CheckObject("\tcat=", ErrMsg.CAT_SLOT_ERR, 
            ErrMsg.CAT_FILLER_ERR, CheckSt.CHECK_CAT_ENTRY, null, 
            new CheckFormatCat());
        // acronym=
        acronymNextStartStrs_.add("\tabbreviation_of=");
        acronymNextStartStrs_.add("\tclass_type=");
        acronymNextStartStrs_.add("annotation=");
        acronymNextStartStrs_.add("signature=");
        acronymNextStartStrs_.add("}");
        checkAcronym_ = new CheckObject("\tacronym_of=", 
            ErrMsg.ACRONYM_SLOT_ERR, ErrMsg.ACRONYM_FILLER_ERR, 
            CheckSt.CHECK_ABBREVIATION, acronymNextStartStrs_, 
            new CheckFormatAcronym());
        // abbreviation=
        abbreviationNextStartStrs_.add("\tclass_type=");
        abbreviationNextStartStrs_.add("annotation=");
        abbreviationNextStartStrs_.add("signature=");
        abbreviationNextStartStrs_.add("}");
        checkAbbreviation_ = new CheckObject("\tabbreviation_of=", 
            ErrMsg.ABBREVIATION_SLOT_ERR, ErrMsg.ABBREVIATION_FILLER_ERR, 
            CheckSt.CHECK_CLASS_TYPE, abbreviationNextStartStrs_, 
            new CheckFormatAbbreviation());
        // slot class_type=
        classTypeNextStartStrs_.add("annotation=");
        classTypeNextStartStrs_.add("signature=");
        classTypeNextStartStrs_.add("}");
        checkClassType_ = new CheckObject("\tclass_type=",
            ErrMsg.CLASS_TYPE_SLOT_ERR, ErrMsg.CLASS_TYPE_FILLER_ERR,
            CheckSt.CHECK_ANNOTATION, classTypeNextStartStrs_, 
			new CheckFormatClassType());
        // annotation=
        annotationNextStartStrs_.add("signature=");
        annotationNextStartStrs_.add("}");
        checkAnnotation_ = new CheckObject("annotation=", 
            ErrMsg.ANNOTATION_SLOT_ERR, ErrMsg.NONE, CheckSt.CHECK_SIGNATURE, 
            annotationNextStartStrs_, null);
        // slot signature=
        signatureNextStartStrs_.add("}");
        checkSignature_ = new CheckObject("signature=", 
            ErrMsg.SIGNATURE_SLOT_ERR, ErrMsg.NONE, CheckSt.CHECK_END, 
            signatureNextStartStrs_, null);
        // slot }
        checkEnd_ = new CheckObject("}", ErrMsg.END_CODE_ERR, 
            ErrMsg.NONE, CheckSt.CHECK_BASE, null, null);
    }
}
