package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class check state for LexCheck.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-58: chlu, 11/16/18, add a new field class_type - check syntax
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckSt
{
    // public constructor
    /**
    *    Creates a default CheckSt object
    */
    public CheckSt()
    {
    }
    /**
    *    Creates a default CheckSt object by specifing state
	*
	* @param state current state of checking status
    */
    public CheckSt(int state)
    {
        curSt_ = state;
        lastSt_ = state;
    }
    // public method
    /**
    * Get the current state of CheckSt object
    *
    * @return  the current state
    */
    public int GetCurState()
    {
        return curSt_;
    }
    /**
    * Get the last state of CheckSt object
    *
    * @return  the last state
    */
    public int GetLastState()
    {
        return lastSt_;
    }
    /**
    * Set the current state of CheckSt object
    *
    * @param  curSt current state to be set
    */
    public void SetCurState(int curSt)
    {
        curSt_ = curSt;
    }
    /**
    * Set the last state of CheckSt object
    *
    * @param  lastSt last state to be set
    */
    public void SetLastState(int lastSt)
    {
        lastSt_ = lastSt;
    }
    /**
    * Increase the value of cuurent state by one to move to the next state
    */
    public void IncreaseState()
    {
        curSt_++;
    }
    /**
    * Update the value of last state by assigning current state to it. 
    */
    public void UpdateLastState()
    {
        lastSt_ = curSt_;
    }
    /**
    * Update cuurent state
    *
    * @param  newSt value of state to be updated
    */
    public void UpdateCurState(int newSt)
    {
        // update cur state
        if(newSt == CheckSt.NO_CHANGE)
        {
            // do nothing
        }
        else if(newSt == CheckSt.INCREASE_ST)
        {
            curSt_++;
        }
        else
        {
            curSt_ = newSt;
        }
    }
    // final data
    /** state: no change */
    final public static int NO_CHANGE = 0;
    /** state: increase state value by one */
    final public static int INCREASE_ST = 1;
    /** state: check base */
    final public static int CHECK_BASE = 10;
    /** state: check spelling variants */
    final public static int CHECK_SPELLING_VAR = 20;
    /** state: check entry (EUI) */
    final public static int CHECK_ENTRY = 21;
    /** state: check category */
    final public static int CHECK_CAT = 30;
    /** state: check category entry */
    final public static int CHECK_CAT_ENTRY = 40;
    /** state: check agreement */
    final public static int CHECK_AGREEMENT = 41;
    /** state: check position */
    final public static int CHECK_POSITION = 42;
    /** state: check complement */
    final public static int CHECK_COMPLEMENT = 43;
    /** state: check stative status */
    final public static int CHECK_STATIVE = 44;
    /** state: check nominalization */
    final public static int CHECK_NOMINALIZATION = 45;
    /** state: check  proper noun */
    final public static int CHECK_PROPER_NOUN = 46;
    /** state: check gender */
    final public static int CHECK_GENDER = 47;
    /** state: check interrogative */
    final public static int CHECK_INTEROATIVE = 48;
    /** state: check type */
    final public static int CHECK_TYPE = 49;
    /** state: check modification type */
    final public static int CHECK_MODIFICATION_TYPE = 50;
    /** state: check state of negative */
    final public static int CHECK_NEGATIVE = 51;
    /** state: check deictic */
    final public static int CHECK_DEICTIC = 52;
    /** state: check acronym */
    final public static int CHECK_ACRONYM = 91;
    /** state: check abbreviation */
    final public static int CHECK_ABBREVIATION = 92;
    /** state: check class types */
    final public static int CHECK_CLASS_TYPE = 93;
    /** state: check annotation */
    final public static int CHECK_ANNOTATION = 95;
    /** state: check signature */
    final public static int CHECK_SIGNATURE = 96;
    /** state: check end bracket */
    final public static int CHECK_END = 99;
    /** state: check adjective */
    final public static int CHECK_ADJ = 100;
    /** state: check adjective inflectional variants */
    final public static int CHECK_ADJ_VARIANTS = 101;
    /** state: check adjective position */
    final public static int CHECK_ADJ_POSITION = 102;
    /** state: check adjective complement */
    final public static int CHECK_ADJ_COMPL = 103;
    /** state: check adjective stative */
    final public static int CHECK_ADJ_STATIVE = 104;
    /** state: check adjective nominalization */
    final public static int CHECK_ADJ_NOMINALIZATION = 105;
    /** state: check adverb */
    final public static int CHECK_ADV = 110;
    /** state: check adverb inflectional variants */
    final public static int CHECK_ADV_VARIANTS = 111;
    /** state: check adverb interrogative */
    final public static int CHECK_ADV_INTERROGATIVE = 112;
    /** state: check adverb modification */
    final public static int CHECK_ADV_MODIFICATION = 113;
    /** state: check adverb negative */
    final public static int CHECK_ADV_NEGATIVE = 114;
    /** state: check adverb  broad negative */
    final public static int CHECK_ADV_BROAD_NEGATIVE = 115;
    /** state: check auxiliary */
    final public static int CHECK_AUX = 120;
    /** state: check auxiliary inflectional variants */
    final public static int CHECK_AUX_VARIANT = 121;
    /** state: check complementizer */
    final public static int CHECK_COMPL = 130;
    /** state: check conjunctions */
    final public static int CHECK_CONJ = 140;
    /** state: check determiner */
    final public static int CHECK_DET = 150;
    /** state: check determiner inflectinal variants */
    final public static int CHECK_DET_VARIANTS = 151;
    /** state: check determiner interrogative */
    final public static int CHECK_DET_INTERROGATIVE = 152;
    /** state: check determiner demonstrative */
    final public static int CHECK_DET_DEMONSTRATIVE = 153;
    /** state: check modal */
    final public static int CHECK_MODAL = 160;
    /** state: check modal inflectional variants */
    final public static int CHECK_MODAL_VARIANT = 161;
    /** state: check noun */
    final public static int CHECK_NOUN = 170;
    /** state: check noun inflectional variants */
    final public static int CHECK_NOUN_VARIANTS = 171;
    /** state: check noun complement */
    final public static int CHECK_NOUN_COMPL = 172;
    /** state: check noun nominalization */
    final public static int CHECK_NOUN_NOMINALIZATION = 173;
    /** state: check noun proper */
    final public static int CHECK_NOUN_PROPER = 174;
    /** state: check noun tradename */
    final public static int CHECK_NOUN_TRADENAME = 175;
    /** state: check noun trade mark */
    final public static int CHECK_NOUN_TRADEMARK = 176;
    /** state: check preposition */
    final public static int CHECK_PREP = 180;
    /** state: check pronoun */
    final public static int CHECK_PRON = 190;
    /** state: check pronoun inflectional variants */
    final public static int CHECK_PRON_VARIANTS = 191;
    /** state: check pronoun gender */
    final public static int CHECK_PRON_GENDER = 192;
    /** state: check pronoun interrogative */
    final public static int CHECK_PRON_INTERROGATIVE = 193;
    /** state: check pronoun type */
    final public static int CHECK_PRON_TYPE = 194;
    /** state: check verb */
    final public static int CHECK_VERB = 200;
    /** state: check verb inflectional variants */
    final public static int CHECK_VERB_VARIANTS = 201;
    /** state: check verb intransitive complement (blank) */
    final public static int CHECK_VERB_INTRAN = 202;
    /** state: check verb intransitive complement */
    final public static int CHECK_VERB_INTRAN2 = 203;
    /** state: check verb transitive complement */
    final public static int CHECK_VERB_TRAN = 204;
    /** state: check verb ditransitive complement */
    final public static int CHECK_VERB_DITRAN = 205;
    /** state: check verb linking complement */
    final public static int CHECK_VERB_LINK = 206;
    /** state: check verb complex transitive */
    final public static int CHECK_VERB_CPLXTRAN = 207;
    /** state: check verb nominolization */
    final public static int CHECK_VERB_NOMINALIZATION = 208;
    // data member
    private int curSt_ = CHECK_BASE;
    private int lastSt_ = CHECK_BASE;
}
