package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class provides a Java class for determiner entry object.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class DetEntry
{
    /**
    *    Creates a default determiner entry object
    */
    public DetEntry()
    {
        variants_ = null;
        interrogative_ = false;
        demonstrative_ = false;
    }
    // public method
    /**
    * Get the agreement variants of an determiner entry
    *
    * @return  the agreement variants in a String format
    */
    public String GetVariants()
    {
        return variants_;
    }
    /**
    * Get the status of interrogative of an determiner entry
    *
    * @return  the status of interrogative
    */
    public boolean IsInterrogative()
    {
        return interrogative_;
    }
    /**
    * Get the status of demonstrative of an determiner entry
    *
    * @return  the status of demonstrative
    */
    public boolean IsDemonstrative()
    {
        return demonstrative_;
    }
    /**
    * Set the aggreement variants of an determiner entry
    *
    * @param  variant variants to be set.
    */
    public void SetVariants(String variant)
    {
        variants_ = variant;
    }
    /**
    * Set the status of interrogative of an determiner entry
    *
    * @param  interrogative status of interrogative to be set
    */
    public void SetInterrogative(boolean interrogative)
    {
        interrogative_ = interrogative;
    }
    /**
    * Set the status of demonstrative of an determiner entry
    *
    * @param  demonstrative status of demonstrative to be set
    */
    public void SetDemonstrative(boolean demonstrative)
    {
        demonstrative_ = demonstrative;
    }
    /**
    * Get the text format of the demonstrative entry
    *
    * @return  text format of the demonstrative entry
    */
    public String GetText()
    {
        String text = new String();
        text = TextLib.AddToText(text, "variants=", variants_, 1);
        text = TextLib.AddToText(text, "interrogative", interrogative_, 1);
        text = TextLib.AddToText(text, "demonstrative", demonstrative_, 1);
        return text;
    }
    /**
    * Get the xml format of the demonstrative entry
    *
    * @return  xml format of the demonstrative entry
    */
    public String GetXml()
    {
        boolean convertFlag = true;
        String xml = new String();
        xml = XmlLib.AddToXml(xml, "<detEntry>", 2); 
        xml = XmlLib.AddToXml(xml, "<variants>", "</variants>", variants_, 
            3, convertFlag);
        xml = XmlLib.AddToXml(xml, "<interrogative/>", interrogative_, 3);
        xml = XmlLib.AddToXml(xml, "<demonstrative/>", demonstrative_, 3);
        xml = XmlLib.AddToXml(xml, "</detEntry>", 2); 
        return xml;
    }
    // data member
    private String variants_ = null;
    private boolean interrogative_ = false;
    private boolean demonstrative_ = false;
}
