package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class provides a Java class for dictionary variants.
*
* <p><b>History:</b>
*
* @author chlu
*
* @version    V-2019
****************************************************************************/
public class DicVar
{
    /**
    * Creates a default dictionary variants by specifying a line with 8 fields,
	* including inflectional variant, inflection, category, eui, unigque,
	* AcrAbb, properNounFlag, and spVarFlag
    *
    * @param  line     inflectional variant
    */
    public DicVar(String line)
	{
		var_ = line.split("\\|")[0];
		cat_ = line.split("\\|")[1];
		infl_ = line.split("\\|")[2];
		eui_ = line.split("\\|")[3];
		unique_ = Boolean.parseBoolean(line.split("\\|")[4]);
		acrAbbFlag_ = Boolean.parseBoolean(line.split("\\|")[5]);
		properNounFlag_ = Boolean.parseBoolean(line.split("\\|")[6]);
		spVarFlag_ = Boolean.parseBoolean(line.split("\\|")[7]);
	}
    /**
    * Creates a default dictionary variants by specifying inflectional variant,
	* inflection, category, eui, AcrAbb, properNounFlag, and spVarFlag
    *
    * @param  var     inflectional variant
    * @param  cat     category
    * @param  infl    inflection
    * @param  eui  unique entry identifier
    * @param  unique  unique flag, true if first count, false if duplicates
    * @param  acrAbbFlag     boolean flag for acrAbb
    * @param  properNounFlag     boolean flag for properNoun
    * @param  spVarFlag     boolean flag for spelling variants
    */
    public DicVar(String var, String cat, String infl, String eui,
		boolean unique, boolean acrAbbFlag, boolean properNounFlag,
		boolean spVarFlag)
    {
        var_ = var;
        cat_ = cat;
        infl_ = infl;
        eui_ = eui;
		unique_ = unique;
        acrAbbFlag_ = acrAbbFlag;
        properNounFlag_ = properNounFlag;
        spVarFlag_ = spVarFlag;
    }
    // public method
    /**
    * Get inflectional variant
    *
    * @return   inflectional variant
    */
    public String GetVar()
    {
        return var_;
    }
    /**
    * Get category
    *
    * @return category name  
    */
    public String GetCat()
    {
        return cat_;
    }
    /**
    * Get inflection
    *
    * @return   inflection
    */
    public String GetInflection()
    {
        return infl_;
    }
    /**
    * Get Eui
    *
    * @return Eui (Entry Unique Identifier)  
    */
    public String GetEui()
    {
        return eui_;
    }
	/**
	* Get unique flag. Inherited from InflVars. It is used to identify if a
	* var is unique (not duplicated from different inflectional rule).
	* The unique flag is false when the inflection variants,
	* eui, category, inflection, uninflection, citation are the same but the
	* type are different.
	*
	* @return   unique flag
	*/
	public boolean GetUnique()
	{
		return unique_;
	}
    /**
    * Get Acronym/Abbreviation flag
    *
    * @return true if vars is a acronym or abbreviation, false otherwise
    */
    public boolean GetAcrAbbFlag()
    {
        return acrAbbFlag_;
    }
    /**
    * Get proper noun flag
    *
    * @return true if it is a proer noun
    */
    public boolean GetProperNounFlag()
    {
        return properNounFlag_;
    }
    /**
    * Get spVar flag
    *
    * @return  true if it has spellign variants
    */
    public boolean GetSpVarFlag()
    {
        return spVarFlag_;
    }
    /**
    * Set the inflectional variant
    *
    * @param   var   inflectional variant
    */
    public void SetVar(String var)
    {
        var_ = var;
    }
    /**
    * Set the category
    *
    * @param   cat catogory 
    */
    public void SetCat(String cat)
    {
        cat_ = cat;
    }
    /**
    * Set the eui
    *
    * @param   eui   entry unque identifier 
    */
    public void SetEui(String eui)
    {
        eui_ = eui;
    }
    /**
    * Set inflectional
    *
    * @param   infl   inflectional
    */
    public void SetInflection(String infl)
    {
        infl_ = infl;
    }
	/**
	* Set the unique flag
	*
	* @param   unique   unique flag
	*/
	public void SetUnique(boolean unique)
	{
		unique_ = unique;
	}
    /**
    * Set the acronym/abbreviation flag
    *
    * @param   acrAbbFlag   acronym/abbreviation flag
    */
    public void SetAcrAbbFlag(boolean acrAbbFlag)
    {
        acrAbbFlag_ = acrAbbFlag;
    }
    /**
    * Set the properNoun flag
    *
    * @param   properNounFlag   proper noun flag
    */
    public void SetProperNounFlag(boolean properNounFlag)
    {
        properNounFlag_ = properNounFlag;
    }
    /**
    * Set the spVar flag
    *
    * @param   spVarFlag   spelliong variant flag
    */
    public void SetSpVarFlag(boolean spVarFlag)
    {
        spVarFlag_ = spVarFlag;
    }
    // data member
    private String var_ = null;            // inflectional variant
    private String cat_ = null;            // category
    private String infl_ = null;        // inflection
    private String eui_ = null;            // uninflection - eui
	private boolean unique_ = true;        // unique vars disgard type
    private boolean acrAbbFlag_ = false;        // acronym/abbreviation
    private boolean properNounFlag_ = false;    // properNoun
    private boolean spVarFlag_ = false;    // has spelling variants
}
