package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class provides a Java class for inflection and inflectional variants.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class InflVar
{
    /**
    * Creates a default inflectional variants by specifing eui, uninflected 
    * term, inflection, uninflectional variant, category, and citation form. 
    *
    * @param  eui  unique entry identifier
    * @param  unInfl  uninflectional (base) form
    * @param  infl    inflection
    * @param  var     inflectional variant
    * @param  cat     category
    * @param  cit     citation form (the citated base form)
    */
    public InflVar(String eui, String unInfl, String infl, String var,
        String cat, String cit)
    {
        eui_ = eui;
        unInfl_ = unInfl;
        type_ = "basic";
        infl_ = infl;
        var_ = var;
        cat_ = cat;
        cit_ = cit;
    }
    /**
    * Creates a default inflectional variants by specifing eui, uninflected 
    * term, type, inflection, uninflectional variant, category, and citation 
    * form. 
    *
    * @param  eui  unique entry identifier
    * @param  unInfl  uninflectional (base) form
    * @param  type    inflection rule type
    * @param  infl    inflection
    * @param  var     inflectional variant
    * @param  cat     category
    * @param  cit     citation form (the citated base form)
    */
    public InflVar(String eui, String unInfl, String type, String infl,
        String var, String cat, String cit)
    {
        eui_ = eui;
        unInfl_ = unInfl;
        type_ = type;
        infl_ = infl;
        var_ = var;
        cat_ = cat;
        cit_ = cit;
    }
    // public method
    /**
    * Get category
    *
    * @return category name  
    */
    public String GetCat()
    {
        return cat_;
    }
    /**
    * Get citation
    *
    * @return citation form  
    */
    public String GetCit()
    {
        return cit_;
    }
    /**
    * Get Eui
    *
    * @return Eui (Entry Unique Identifier)  
    */
    public String GetEui()
    {
        return eui_;
    }
    /**
    * Get uninflectional variant
    *
    * @return uninflectional variant
    */
    public String GetUnInfl()
    {
        return unInfl_;
    }
    /**
    * Get inflectional rule type
    *
    * @return the type of inflection rule
    */
    public String GetType()
    {
        return type_;
    }
    /**
    * Get inflection
    *
    * @return   inflection
    */
    public String GetInflection()
    {
        return infl_;
    }
    /**
    * Get inflectional variant
    *
    * @return   inflectional variant
    */
    public String GetVar()
    {
        return var_;
    }
    /**
    * Get unique flag.  The unique flag is false when the inflection variants,
    * eui, category, inflection, uninflection, citation are the same but the
    * type are different.
    *
    * @return   unique flag
    */
    public boolean GetUnique()
    {
        return unique_;
    }
    /**
    * Set the category
    *
    * @param   cat catogory 
    */
    public void SetCat(String cat)
    {
        cat_ = cat;
    }
    /**
    * Set the citation
    *
    * @param   cit citation 
    */
    public void SetCit(String cit)
    {
        cit_ = cit;
    }
    /**
    * Set the eui
    *
    * @param   eui   entry unque identifier 
    */
    public void SetEui(String eui)
    {
        eui_ = eui;
    }
    /**
    * Set the uninflectional form (base)
    *
    * @param   unInfl   uninflectional form (base) 
    */
    public void SetUnInfl(String unInfl)
    {
        unInfl_ = unInfl;
    }
    /**
    * Set the type of inflectional rule
    *
    * @param   type   type of inflectional rule
    */
    public void SetType(String type)
    {
        type_ = type;
    }
    /**
    * Set inflectional
    *
    * @param   infl   inflectional
    */
    public void SetInflection(String infl)
    {
        infl_ = infl;
    }
    /**
    * Set the inflectional variant
    *
    * @param   var   inflectional variant
    */
    public void SetVar(String var)
    {
        var_ = var;
    }
    /**
    * Set the unique flag
    *
    * @param   unique   unique flag
    */
    public void SetUnique(boolean unique)
    {
        unique_ = unique;
    }
    // data member
    private String var_ = null;            // inflectional variant
    private String cat_ = null;            // category
    private String infl_ = null;        // inflection
    private String eui_ = null;            // uninflection - eui
    private String unInfl_ = null;        // uninflection - base
    private String cit_ = null;            // citation
    private String type_ = null;        // inflection rule type
    private boolean unique_ = true;        // unique vars disgard type
}
