package gov.nih.nlm.nls.lexCheck.Tools;
import java.util.*;
import java.io.*;
import java.nio.file.*;
import java.nio.charset.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Gram.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
import gov.nih.nlm.nls.lvg.Lib.*;
/*****************************************************************************
* This class provides a command line program to generate inflected 
* variants from a set of lexical records (text format) read in from a file.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class GenerateInflVars
{
    // public method
    /**
    * command line program for generating inflected variants from
    * a input file.
	*
	* @param  args  input arguments
    */
    public static void main(String[] args)
    {
		// from optin String if there are arguments
		String inFile = "LEXICON";
		String outFile = "inflVars.data";
		boolean verbose = false;
        if(args.length == 3)
        {
			inFile = args[0];
			outFile = args[1];
			if(args[2].equals("-v") == true)
			{
				verbose = true;
			}
		}
        else if(args.length == 2)
        {
			inFile = args[0];
			outFile = args[1];
		}
		else if(args.length >= 0)
		{
            System.out.println("** Usage: java GenerateInflVars <inFile> <outFile> <-v:verbose>");
            System.exit(0);
        }

		// print out
		if(verbose == true)
		{
			System.out.println("-- inFile: [" + inFile + "]");
			System.out.println("-- outFile: [" + outFile + "]");
		}
		try
		{
			// Go through LexCheck
			// using ToInflVarsApi
			Vector<InflVar> inflVars 
				= ToInflVarsApi.GetInflVarsFromTextFile(inFile);
			InflVarComparator<InflVar> comp = new InflVarComparator<InflVar>();
			Collections.sort(inflVars, comp);
			BufferedWriter outWriter = Files.newBufferedWriter(
				Paths.get(outFile), Charset.forName("UTF-8"));
			// print out result
			for(int i = 0; i < inflVars.size(); i++)
			{
				InflVar inflVar = inflVars.elementAt(i);
				// remove the duplicate inflVars
				if(inflVar.GetUnique() == true)
				{
					String outStr = inflVar.GetVar() + "|" 
						+ Category.ToValue(inflVar.GetCat()) + "|"
						+ Inflection.ToValue(inflVar.GetInflection()) + "|"
						+ inflVar.GetEui() + "|"
						+ inflVar.GetUnInfl() + "|"
						+ inflVar.GetCit();
					outWriter.write(outStr);
					outWriter.newLine();
				}
			}
			// close
			outWriter.close();
		}
		catch(Exception x)
		{
			System.out.println("** ERR@GenerateInflVars( ): " + x.toString());
		}
    }
}
