package gov.nih.nlm.nls.lexCheck.Tools;
import com.ibm.xml.parser.Parser;
import java.io.*;
import java.util.*;
import java.nio.file.*;
import java.nio.charset.*;
import java.util.Hashtable;
import org.w3c.dom.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lvg.Lib.*;
/*****************************************************************************
* This class provides a command line program for generating java object of 
* a lexical record which reads in it's Text format from a file.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-59, chlu, 12-05-18, enhance the algorithm on args
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class ToJavaObjectFromTextFile
{
    /**
    * command line program for generating java object of a lexical record 
    * by reading a Text foramt from a input file.
	*
	* @param  args  input arguments
    */
    static public void main(String[] args)
    {
		// init args
		String inFile = "../data/Files/Test.data";
		String txtFile = null;
		String inflVarsFile = null;
		boolean verbose = false;
        if(args.length == 4)
		{
			inFile = args[0];
			txtFile = args[1];
			inflVarsFile = args[2];
			if(args[3].equals("-v") == true)
			{
				verbose = true;
			}
		}
        else if(args.length == 3)
		{
			inFile = args[0];
			txtFile = args[1];
			inflVarsFile = args[2];
		}
        else if(args.length == 2)
		{
			inFile = args[0];
			txtFile = args[1];
		}
		else if(args.length >= 0)
        {
            System.err.println("** Usage: java ToJavaObjectFromTextFile <inFile(Text)> <outLexiconFile> <outInflVarsFile> <-v: verbose>");
            System.exit(1);
        }

		// print out
		if(verbose == true)
		{
			System.out.println("----- ToJavaObjectFromTextFile -----");
			System.out.println("1. in lexiconFile: [" + inFile + "]");
			System.out.println("2. out txtFile: [" + txtFile + "]");
			System.out.println("3. out inflVarsFile: [" + inflVarsFile + "]");
		}

		// process ToJavaObj
        try
        {
			BufferedWriter tWriter = Files.newBufferedWriter(
				Paths.get(txtFile), Charset.forName("UTF-8"));
			BufferedWriter iWriter = null;
			if(inflVarsFile != null)
			{
				iWriter = Files.newBufferedWriter(
					Paths.get(inflVarsFile), Charset.forName("UTF-8"));
				if(verbose == true)
				{
					System.out.println("----- Print Inflection Vars -----");
				}
			}

			// read the inFile and convert to JavaObj of lexRecords
            Vector<LexRecord> lexRecords 
                = ToJavaObjApi.ToJavaObjsFromTextFile(inFile);
            if(lexRecords.size() <= 0)
            {
                System.exit(1);
            }
            else
            {
				InflVarComparator<InflVar> comp 
					= new InflVarComparator<InflVar>();
                // print out lexRecords
                for(int i = 0; i < lexRecords.size(); i++)
                {
                    LexRecord lexRecord = lexRecords.elementAt(i);

					// print out lexRecords in Text
                    tWriter.write(lexRecord.GetText());

                    // print out inflVars
                    if(inflVarsFile != null)
                    {
						// get inflVars of each lexRecord
                        Vector<InflVar> inflVars = 
                        lexRecord.GetInflVarsAndAgreements().GetInflValues();
						Collections.sort(inflVars, comp);

                        for(int j = 0; j < inflVars.size(); j++)
                        {
                            InflVar inflVar = inflVars.elementAt(j);
							// remove the duplicate inflVars
							if(inflVar.GetUnique() == true)
							{
								iWriter.write(inflVar.GetVar() + "|"
									+ Category.ToValue(inflVar.GetCat()) + "|"
									+ Inflection.ToValue(inflVar.GetInflection()) 
									+ "|"
									+ inflVar.GetEui() + "|"
									+ inflVar.GetUnInfl() + "|"
									+ inflVar.GetCit());
								iWriter.newLine();	
							}
                        }
                    }
                }
            }

			// close
			tWriter.close();
			if(inflVarsFile != null)
			{
				iWriter.close();
			}
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
}
