package gov.nih.nlm.nls.lexCheck.Tools;
import java.util.*;
import java.io.*;
import java.nio.file.*;
import java.nio.charset.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Gram.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a command line program for transferring lexical record 
* to a out file in XML format.  The input is the filename of the file contains 
* lexical records in text format.  This program ustilizes ToXmlApi and have 
* same major functions as lexCheck.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-48, chlu, 12-11-18, check particles in verb particle constructions
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class ToXmlFromTextFile
{
    // public method
    /**
    * command line program for transferring lexical record from it's text to 
    * XML format.
	*
	* @param  args  input arguments
    */
    public static void main(String[] args)
    {
        String inFile = "LEXICON";
        String outFile = "LEXICON.xml";
        String prepositionFile = "../data/Files/prepositions.data";
        String particleFile = "../data/Files/particles.data";
        boolean verbose = false;
        // from optin String if there are arguments
        if(args.length == 5)
        {
            inFile = args[0];
            outFile = args[1];
            prepositionFile = args[2];
            particleFile = args[3];
            if(args[4].equals("-v") == true)
            {
                verbose=true;
            }
        }
        else if(args.length == 4)
        {
            inFile = args[0];
            outFile = args[1];
            prepositionFile = args[2];
            particleFile = args[3];
        }
        else if(args.length == 2)
        {
            inFile = args[0];
            outFile = args[1];
        }
        else if(args.length >= 0)
        {
            System.err.println("** Usage: java ToXmlFromTextFile <inFile> <outFile> <prepositionFile> <particleFile> <verbose>");
            System.exit(1);
        }
        // set preposition
        GlobalVars.SetPrepositionFile(prepositionFile);
        GlobalVars.SetParticleFile(particleFile);
        
        // verbose
        if(verbose == true)
        {
            System.out.println("===== ToXmlFromTextFile.( ) ======");
            System.out.println("-- inFile: [" + inFile + "]");
            System.out.println("-- outFile: [" + outFile + "]");
            System.out.println("-- prepositionsFile: ["
                + GlobalVars.GetPrepositionFile() + "]");
            System.out.println("-- particleFile: ["
                + GlobalVars.GetParticleFile() + "]");
        }

        // Go through LexCheck
		try
		{
			BufferedWriter oWriter = Files.newBufferedWriter(
				Paths.get(outFile), Charset.forName("UTF-8"));
				
			// print out: Use ToXmlApi with ApiOutput
        	oWriter.write(ToXmlApi.ToXmlFromTextFile(inFile).GetXml());
			oWriter.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
    }
}
